Package: LFApp
Version: 1.4.2
Date: 2026-05-05
Title: Shiny Apps for Lateral Flow Assays
Authors@R: c(person("Filip", "Paskali", role = c("aut", "cre"),
                  email = "Filip.Paskali@gmail.com",
                  comment = c(ORCID = "0000-0002-9647-6294")),
             person("Weronika", "Schary", role = "aut",
                  email = "Weronika.Schary@hs-furtwangen.de", 
                  comment = c(ORCID = "0000-0002-7229-316X")),
             person("Matthias", "Kohl", role = "aut",
                  email = "Matthias.Kohl@stamats.de", 
                  comment = c(ORCID = "0000-0001-9514-8910")))
Author: Filip Paskali [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-9647-6294>),
  Weronika Schary [aut] (ORCID: <https://orcid.org/0000-0002-7229-316X>),
  Matthias Kohl [aut] (ORCID: <https://orcid.org/0000-0001-9514-8910>)
Maintainer: Filip Paskali <Filip.Paskali@gmail.com>
Description: Shiny apps for the quantitative analysis of images from lateral flow assays (LFAs). The images are segmented and background corrected and color intensities are extracted. The apps can be used to import and export intensity data and to calibrate LFAs by means of linear, loess, or gam models. The calibration models can further be saved and applied to intensity data from new images for determining concentrations.
License: LGPL-3
Depends: R (>= 4.0.0)
Imports: stats, utils, graphics, methods, mgcv, shiny, shinyjs,
        shinythemes, shinyFiles, shinyMobile (>= 0.9), EBImage, DT,
        ggplot2, fs
Suggests: knitr, rmarkdown, remotes
VignetteBuilder: knitr
Encoding: UTF-8
URL: https://github.com/fpaskali/LFApp
BugReports: https://github.com/fpaskali/LFApp/issues
NeedsCompilation: no
Packaged: 2026-05-05 18:56:18 UTC; paf
Repository: CRAN
Date/Publication: 2026-05-05 19:20:18 UTC
Built: R 4.6.0; ; 2026-05-05 21:39:16 UTC; unix

importFrom("stats", "as.formula", "lm", "loess", "median", "quantile")
importFrom("utils", "browseURL", "read.csv", "write.csv")
importFrom("graphics", "hist.default", "lines", "par", "title")
importFrom("methods", "is")
importFrom("mgcv", "gam")
import("shiny")
importFrom("shinyjs", "useShinyjs")
importFrom("shinythemes", "shinytheme")
importFrom("shinyFiles", "parseDirPath")
import("shinyMobile")
importFrom("EBImage", "colorMode", "otsu", "readImage", "channel", "imageData")
importFrom("DT", "DTOutput", "datatable", "renderDT")
importFrom("ggplot2", "annotate", "geom_abline", "geom_point", "ggplot", "ylab")
importFrom("fs", "dir_create", "path_home")
export(run_analysis, 
       run_cal,
       run_core,
       run_quan,
       run_mobile_analysis, 
       run_mobile_cal,
       run_mobile_core,
       run_mobile_quan,  
       threshold_li, 
       triangle)

## ----eval = FALSE-------------------------------------------------------------
# ## Install package BiocManager
# if(!requireNamespace("BiocManager", quietly = TRUE))
#   install.packages("BiocManager")
# ## Use BiocManager to install EBImage
# BiocManager::install("EBImage", update = FALSE)

## ----eval = FALSE-------------------------------------------------------------
# ## Install package knitr
# if(!requireNamespace("knitr", quietly = TRUE))
#   install.packages("knitr")
# ## Install package rmarkdown
# if(!requireNamespace("rmarkdown", quietly = TRUE))
#   install.packages("rmarkdown")

## ----eval = FALSE-------------------------------------------------------------
# ## Install package remotes
# if(!requireNamespace("remotes", quietly = TRUE))
#   install.packages("remotes")
# ## Install package LFApp
# remotes::install_github("fpaskali/LFApp", build_vignette=TRUE)

## ----eval = FALSE-------------------------------------------------------------
# ## desktop versions
# ## LFA App core
# LFApp::run_core()
# 
# ## LFA App quantification
# LFApp::run_quan()
# 
# ## LFA App calibration
# LFApp::run_cal()
# 
# ## LFA App full analysis
# LFApp::run_analysis()
# 
# ## mobile versions
# ## LFA App core
# LFApp::run_mobile_core()
# 
# ## LFA App quantification
# LFApp::run_mobile_quan()
# 
# ## LFA App calibration
# LFApp::run_mobile_cal()
# 
# ## LFA App full analysis
# LFApp::run_mobile_analysis()

## desktop versions
run_analysis <- function(){
  shinyApp(ui = analysis_ui, server = analysis_server)
}
run_cal <- function(){
  shinyApp(ui = cal_ui, server = cal_server)
}
run_core <- function(){
  shinyApp(ui = core_ui, server = core_server)
}
run_quan <- function(){
  shinyApp(ui = quan_ui, server = quan_server)
}

## mobile versions
run_mobile_analysis <- function(){
  shinyApp(ui = analysis_mobile_ui, server = analysis_mobile_server)
}
run_mobile_cal <- function(){
  shinyApp(ui = cal_mobile_ui, server = cal_mobile_server)
}
run_mobile_core <- function(){
  shinyApp(ui = core_mobile_ui, server = core_mobile_server)
}
run_mobile_quan <- function(){
  shinyApp(ui = quan_mobile_ui, server = quan_mobile_server)
}

quan_mobile_server <- function(input, output, session){
  ########## FIRST TAB
  oldpar <- par(no.readonly = TRUE)
  on.exit(par(oldpar))

  oldopt <- options()
  on.exit(options(oldopt))
  options(shiny.maxRequestSize=100*1024^2) #file can be up to 50 mb; default is 5 mb
  shinyImageFile <- reactiveValues(shiny_img_origin = NULL, shiny_img_cropped = NULL,
                                   shiny_img_final = NULL, Threshold = NULL)
  IntensData <- NULL
  calFun <- NULL
  quanData <- NULL
  predictData <- NULL

  # checks upload for file imput
  observe({
    #default: upload image
    if(input$upload == "Upload image"){
      output$plot1 <- renderPlot({
        if(is.null(input$file1))
          return(NULL)
        validate(need(!is.null(input$file1), "Must upload a valid jpg, png, or tiff"))
      })
    }
    if(input$upload == "Sample"){
      # using sample image
      img <- readImage(system.file("images", "sample.TIF", package="LFApp"))
      shinyImageFile$shiny_img_origin <- img
      shinyImageFile$shiny_img_cropped <- img
      shinyImageFile$shiny_img_final <- img

      shinyImageFile$filename <- "sample.TIF"
      #outputs image to plot1 -- main plot
      output$plot1 <- renderPlot({ EBImage::display(shinyImageFile$shiny_img_final, method = "raster") })
    }
    drawCropButtons()
  })

  # if the new file is entered, it will become new ImageFile
  observeEvent(input$file1, {
    shinyImageFile$filename <- input$file1$name
    img <- readImage(input$file1$datapath)
    shinyImageFile$shiny_img_origin <- img
    shinyImageFile$shiny_img_cropped <- img
    shinyImageFile$shiny_img_final <- img
    output$plot1 <- renderPlot({EBImage::display(img, method = "raster")})
  })

  drawCropButtons <- function(resetButton=FALSE, segButton=FALSE) {
    output$cropButtons <- renderUI({
      tagList(
        f7Segment(
          if (resetButton) {
            f7Button("reset", color = "blue", label = "Reset")
          } else {
            f7Button("no-reset", color = "gray", label = "Reset")
          },
          if (segButton) {
            f7Button("segmentation", color="green", label = "Apply Segmentation")
          } else {
            f7Button("no-segmentation", color="gray", label = "Apply Segmentation")
          }
        )
      )
    })
  }

  # Rotation ----------------------------------------------------------------

  observe({reactiveRotation()})

  reactiveRotation <- eventReactive(input$rotate, {
    isolate({
      if (!is.null(shinyImageFile$shiny_img_cropped)) {
        shinyImageFile$shiny_img_final <- EBImage::rotate(shinyImageFile$shiny_img_cropped, input$rotate,
                                                          bg.col="white")
        output$plot1 <- renderPlot({EBImage::display(shinyImageFile$shiny_img_final, method = "raster")})
        session$resetBrush("plot_brush")
      }
    })
  })

  observe({reactiveRotationCCW()})

  reactiveRotationCCW <- eventReactive(input$rotateCCW, {
    isolate({
      if (!is.null(shinyImageFile$shiny_img_cropped)) {
        shinyImageFile$shiny_img_cropped <- EBImage::rotate(shinyImageFile$shiny_img_cropped, -90)
        shinyImageFile$shiny_img_final <- shinyImageFile$shiny_img_cropped
        output$plot1 <- renderPlot({EBImage::display(shinyImageFile$shiny_img_final, method = "raster")})
        session$resetBrush("plot_brush")
      }
    })
  })

  observe({reactiveRotationCW()})

  reactiveRotationCW <- eventReactive(input$rotateCW, {
    isolate({
      if (!is.null(shinyImageFile$shiny_img_cropped)) {
        shinyImageFile$shiny_img_cropped <- EBImage::rotate(shinyImageFile$shiny_img_cropped, 90)
        shinyImageFile$shiny_img_final <- shinyImageFile$shiny_img_cropped
        output$plot1 <- renderPlot({EBImage::display(shinyImageFile$shiny_img_final, method = "raster")})
        session$resetBrush("plot_brush")
      }
    })
  })

  observe({reactiveRotationFlip()})

  reactiveRotationFlip <- eventReactive(input$fliphor, {
    isolate({
      if (!is.null(shinyImageFile$shiny_img_cropped)) {
        shinyImageFile$shiny_img_cropped <- EBImage::flip(shinyImageFile$shiny_img_cropped)
        shinyImageFile$shiny_img_final <- shinyImageFile$shiny_img_cropped
        output$plot1 <- renderPlot({EBImage::display(shinyImageFile$shiny_img_final, method = "raster")})
        session$resetBrush("plot_brush")
      }
    })
  })

  observe({reactiveRotationFlop()})

  reactiveRotationFlop <- eventReactive(input$flipver, {
    isolate({
      if (!is.null(shinyImageFile$shiny_img_cropped)) {
        shinyImageFile$shiny_img_cropped <- EBImage::flop(shinyImageFile$shiny_img_cropped)
        shinyImageFile$shiny_img_final <- shinyImageFile$shiny_img_cropped
        output$plot1 <- renderPlot({EBImage::display(shinyImageFile$shiny_img_final, method = "raster")})
        session$resetBrush("plot_brush")
      }
    })
  })

  croppedImage <- function(image, xmin, ymin, xmax, ymax){
    if(length(dim(image)) == 2)
      image <- image[xmin:xmax, ymin:ymax, drop = FALSE]
    else if(length(dim(image)) == 3)
      image <- image[xmin:xmax, ymin:ymax, ,drop = FALSE]
    return(image)
  }

  observe({resetImage()})

  resetImage <- eventReactive(input$reset,{
    isolate({
      shinyImageFile$shiny_img_cropped <- shinyImageFile$shiny_img_origin
      shinyImageFile$shiny_img_final <- shinyImageFile$shiny_img_cropped
      output$plot1 <- renderPlot({EBImage::display(shinyImageFile$shiny_img_final, method = "raster")})
      session$resetBrush("plot_brush")
      updateSliderInput(session, "rotate", value=0)
      drawCropButtons()
    })
  })


  #prompts shiny to look at recursive crop
  observe({recursiveCrop()})

  #only executes when selection box is double clicked
  recursiveCrop <- eventReactive(input$plot_dblclick,{
    isolate({
      p <- input$plot_brush
      validate(need(p$xmax <= dim(shinyImageFile$shiny_img_cropped)[1],
                    "Highlighted portion is out of bounds on the x-axis"))
      validate(need(p$ymax <= dim(shinyImageFile$shiny_img_cropped)[2],
                    "Highlighted portion is out of bounds on the y-axis"))
      validate(need(p$xmin >= 0,
                    "Highlighted portion is out of bounds on the x-axis"))
      validate(need(p$ymin >= 0,
                    "Highlighted portion is out of bounds on the y-axis"))
      shinyImageFile$shiny_img_cropped <- croppedImage(shinyImageFile$shiny_img_final, p$xmin, p$ymin, p$xmax, p$ymax)
      shinyImageFile$shiny_img_final <- shinyImageFile$shiny_img_cropped
      output$plot1 <- renderPlot({
        EBImage::display(shinyImageFile$shiny_img_final, method = "raster")
      })
      updateF7Slider("rotate", value=0)
      session$resetBrush("plot_brush")
      drawCropButtons(resetButton = TRUE)
    })
    session$resetBrush("plot_brush")
  })


  observe({recursiveGrid()})

  recursiveGrid <- eventReactive(input$plot_brush,{
    isolate({
      p <- input$plot_brush
      output$plot1 <- renderPlot({
        EBImage::display(shinyImageFile$shiny_img_final, method = "raster")

        colcuts <- seq(p$xmin, p$xmax, length.out = input$strips + 1)
        rowcuts <- seq(p$ymin, p$ymax, length.out = 2*input$bands) # bands + spaces between bands

        for (x in colcuts) {
          lines(x = rep(x, 2), y = c(p$ymin, p$ymax), col="red")
        }
        for (y in rowcuts) {
          lines(x = c(p$xmin, p$xmax), y = rep(y, 2), col="red")
        }
      })
      drawCropButtons(segButton = TRUE, resetButton = TRUE)
    })
  })


  observe({recursiveSegmentation()})

  #only executes when Apply Segmentation is clicked
  recursiveSegmentation <- eventReactive(input$segmentation,{
    isolate({
      p <- input$plot_brush
      # Check if the region of interest is out of the bounds
      if (p$xmax <= dim(shinyImageFile$shiny_img_cropped)[1] &&
          p$ymax <= dim(shinyImageFile$shiny_img_cropped)[2] &&
          p$xmin >= 0 &&
          p$ymin >= 0) {
            MAX <- dim(shinyImageFile$shiny_img_cropped)[1:2]
            colcuts <- seq(p$xmin, p$xmax, length.out = input$strips + 1)
            rowcuts <- seq(p$ymin, p$ymax, length.out = 2*input$bands)

            segmentation.list <- vector("list", length = input$strips)
            count <- 0
            for(i in 1:input$strips){
              tmp.list <- vector("list", length = 2*input$bands-1)
              for(j in 1:(2*input$bands-1)){
                img <- shinyImageFile$shiny_img_final
                if(length(dim(img)) == 2)
                  img <- img[colcuts[i]:colcuts[i+1], rowcuts[j]:rowcuts[j+1]]
                else if(length(dim(img)) == 3)
                  img <- img[colcuts[i]:colcuts[i+1], rowcuts[j]:rowcuts[j+1], , drop = FALSE]
                tmp.list[[j]] <- img
              }
              segmentation.list[[i]] <- tmp.list
            }
            shinyImageFile$cropping_grid <- list("columns" = colcuts, "rows" = rowcuts)
            shinyImageFile$segmentation_list <- segmentation.list
            updateF7Tabs(session=session, id="tabs", selected = "Background")
      } else {
        f7Toast(text="Error: The grid is out of bounds", position="top", session=session)
      }
    })
  })

  ################ END OF FIRST TAB

  ############## SECOND TAB
  observe({
    input$thresh
    updateF7Stepper("selectStrip", max=input$strips)
  })

  showThresholdPlots <- function() {
    output$threshPlots <- renderUI({
      tagList(
        f7Block(
          strong = TRUE,
          inset = FALSE,
          h3('Background Correction', align = "center"),
          verbatimTextOutput("thresh"),
          h4('Signal Intensity Above Background', align = "center"),
          plotOutput("plot3"),
          h4('Lines After Background Subtraction', align = "center"),
          plotOutput("plot4"),
          verbatimTextOutput("meanIntens"),
          verbatimTextOutput("medianIntens"),
          f7Segment(
            f7Button("data", color="green", label = "Add to Data"),
            f7Button("showIntensData", label = "Switch To Intensity Data")
          )
        )
      )
    })
  }

  observe({recursiveThreshold()})

  recursiveThreshold <- eventReactive(input$threshold,{
    isolate({
      showThresholdPlots()
      seg.list <- shinyImageFile$segmentation_list
      i <- input$selectStrip
      if(input$thresh == "Quantile"){
        Background <- vector(mode = "list", length = input$bands)
        for(j in 1:input$bands){
          img <- seg.list[[i]][[j]]
          if(colorMode(img) > 0){
            img <- 1-channel(img, input$channel)
          }
          if(input$invert=="Yes") {
            img <- 1 - img
          }
          Background[[j]] <- as.numeric(EBImage::imageData(img))
        }
        Background.Threshold <- quantile(unlist(Background),
                                         probs = input$quantile1/100)
        shinyImageFile$Threshold <- Background.Threshold
        output$plot3 <- renderPlot({
          par(mfcol = c(1, input$bands))
          Bands <- seq(1, 2*input$bands-1, by = 2)
          count <- 0
          for(j in Bands){
            count <- count + 1
            img <- seg.list[[i]][[j]]
            if(colorMode(img) > 0){
              img <- 1-channel(img, input$channel)
            }
            if(input$invert=="Yes") {
              img <- 1 - img
            }
            signal <- EBImage::imageData(img) > Background.Threshold
            EBImage::imageData(img) <- signal
            plot(img)
            title(paste0("Line ", count))
          }
        })
        shinyImageFile$Mean_Intensities <- matrix(0, nrow = 1, ncol = input$bands)
        shinyImageFile$Median_Intensities <- matrix(0, nrow = 1, ncol = input$bands)
        output$plot4 <- renderPlot({
          par(mfcol = c(1, input$bands))
          Bands <- seq(1, 2*input$bands-1, by = 2)
          count <- 0
          for(j in Bands){
            count <- count + 1
            img <- seg.list[[i]][[j]]
            if(colorMode(img) > 0){
              img <- 1-channel(img, input$channel)
            }
            if(input$invert=="Yes") {
              img <- 1 - img
            }
            signal <- EBImage::imageData(img) > Background.Threshold
            EBImage::imageData(img) <- (EBImage::imageData(img) - Background.Threshold)*signal
            shinyImageFile$Mean_Intensities[1,count] <- mean(EBImage::imageData(img)[signal])
            shinyImageFile$Median_Intensities[1,count] <- median(EBImage::imageData(img)[signal])
            plot(img)
            title(paste0("Line ", count))
          }
        })
      }
      else if(input$thresh == "Otsu"){
        Background.Threshold <- numeric(input$bands)
        output$plot3 <- renderPlot({
          par(mfcol = c(1, input$bands))
          count1 <- 0
          Bands <- seq(1, 2*input$bands-1, by = 2)
          count2 <- 0
          for(j in Bands){
            count1 <- count1 + 1
            count2 <- count2 + 1
            img <- seg.list[[i]][[j]]
            if(colorMode(img) > 0){
              img <- 1-channel(img, input$channel)
            }
            if(input$invert=="Yes") {
              img <- 1 - img
            }
            Background.Threshold[count1] <- otsu(img)
            signal <- EBImage::imageData(img) > Background.Threshold[count1]
            EBImage::imageData(img) <- signal
            plot(img)
            title(paste0("Line ", count2))
          }
          shinyImageFile$Threshold <- Background.Threshold
        })
        shinyImageFile$Mean_Intensities <- matrix(0, nrow = 1, ncol = input$bands)
        shinyImageFile$Median_Intensities <- matrix(0, nrow = 1, ncol = input$bands)
        output$plot4 <- renderPlot({
          par(mfcol = c(1, input$bands))
          count1 <- 0
          Bands <- seq(1, 2*input$bands-1, by = 2)
          count2 <- 0
          for(j in Bands){
            count1 <- count1 + 1
            count2 <- count2 + 1
            img <- seg.list[[i]][[j]]
            if(colorMode(img) > 0){
              img <- 1-channel(img, input$channel)
            }
            if(input$invert=="Yes") {
              img <- 1 - img
            }
            thr <- otsu(img)
            signal <- EBImage::imageData(img) > thr
            EBImage::imageData(img) <- (EBImage::imageData(img) - thr)*signal
            shinyImageFile$Mean_Intensities[1,count1] <- mean(EBImage::imageData(img)[signal])
            shinyImageFile$Median_Intensities[1,count1] <- median(EBImage::imageData(img)[signal])
            plot(img)
            title(paste0("Line ", count2))
          }
        })
      }
      else if(input$thresh == "Triangle"){
        Background.Threshold <- numeric(input$bands)
        output$plot3 <- renderPlot({
          par(mfcol = c(1, input$bands))
          count1 <- 0
          Bands <- seq(1, 2*input$bands-1, by = 2)
          count2 <- 0
          for(j in Bands){
            count1 <- count1 + 1
            count2 <- count2 + 1
            img <- seg.list[[i]][[j]]
            if(colorMode(img) > 0){
              img <- 1-channel(img, input$channel)
            }
            if(input$invert=="Yes") {
              img <- 1 - img
            }
            Background.Threshold[count1] <- triangle(img, input$tri_offset)
            signal <- EBImage::imageData(img) > Background.Threshold[count1]
            EBImage::imageData(img) <- signal
            plot(img)
            title(paste0("Line ", count2))
          }
          shinyImageFile$Threshold <- Background.Threshold
        })
        shinyImageFile$Mean_Intensities <- matrix(0, nrow = 1, ncol = input$bands)
        shinyImageFile$Median_Intensities <- matrix(0, nrow = 1, ncol = input$bands)
        output$plot4 <- renderPlot({
          par(mfcol = c(1, input$bands))
          count1 <- 0
          Bands <- seq(1, 2*input$bands-1, by = 2)
          count2 <- 0
          for(j in Bands){
            count1 <- count1 + 1
            count2 <- count2 + 1
            img <- seg.list[[i]][[j]]
            if(colorMode(img) > 0){
              img <- 1-channel(img, input$channel)
            }
            if(input$invert=="Yes") {
              img <- 1 - img
            }
            thr <- triangle(img, input$tri_offset)
            signal <- EBImage::imageData(img) > thr
            EBImage::imageData(img) <- (EBImage::imageData(img) - thr)*signal
            shinyImageFile$Mean_Intensities[1,count1] <- mean(EBImage::imageData(img)[signal])
            shinyImageFile$Median_Intensities[1,count1] <- median(EBImage::imageData(img)[signal])
            plot(img)
            title(paste0("Line ", count2))
          }
        })
      }
      else if(input$thresh == "Li"){
        Background.Threshold <- numeric(input$bands)
        output$plot3 <- renderPlot({
          par(mfcol = c(1, input$bands))
          count1 <- 0
          Bands <- seq(1, 2*input$bands-1, by = 2)
          count2 <- 0
          for(j in Bands){
            count1 <- count1 + 1
            count2 <- count2 + 1
            img <- seg.list[[i]][[j]]
            if(colorMode(img) > 0){
              img <- 1-channel(img, input$channel)
            }
            if(input$invert=="Yes") {
              img <- 1-img
            }
            Background.Threshold[count1] <- threshold_li(img)
            signal <- EBImage::imageData(img) > Background.Threshold[count1]
            EBImage::imageData(img) <- signal
            plot(img)
            title(paste0("Line ", count2))
          }
          shinyImageFile$Threshold <- Background.Threshold
        })
        shinyImageFile$Mean_Intensities <- matrix(0, nrow = 1, ncol = input$bands)
        shinyImageFile$Median_Intensities <- matrix(0, nrow = 1, ncol = input$bands)
        output$plot4 <- renderPlot({
          par(mfcol = c(1, input$bands))
          count1 <- 0
          Bands <- seq(1, 2*input$bands-1, by = 2)
          count2 <- 0
          for(j in Bands){
            count1 <- count1 + 1
            count2 <- count2 + 1
            img <- seg.list[[i]][[j]]
            if(colorMode(img) > 0){
              img <- 1-channel(img, input$channel)
            }
            if(input$invert=="Yes") {
              img <- 1-img
            }
            thr <- threshold_li(img)
            signal <- EBImage::imageData(img) > thr
            EBImage::imageData(img) <- (EBImage::imageData(img) - thr)*signal
            shinyImageFile$Mean_Intensities[1,count1] <- mean(EBImage::imageData(img)[signal])
            shinyImageFile$Median_Intensities[1,count1] <- median(EBImage::imageData(img)[signal])
            plot(img)
            title(paste0("Line ", count2))
          }
        })
      }
    })
  })

  output$thresh <- renderText({
    if(!is.null(shinyImageFile$Threshold))
      paste0("Threshold(s): ", paste0(signif(shinyImageFile$Threshold, 4), collapse = ", "))
  })
  output$meanIntens <- renderText({
    if(!is.null(shinyImageFile$Threshold))
      paste0("Mean intensities: ", paste0(signif(shinyImageFile$Mean_Intensities, 4), collapse = ", "))
  })
  output$medianIntens <- renderText({
    if(!is.null(shinyImageFile$Threshold))
      paste0("Median intensities: ", paste0(signif(shinyImageFile$Median_Intensities, 4), collapse = ", "))
  })

  observe({recursiveData()})

  recursiveData <- eventReactive(input$data,{
    isolate({
      AM <- shinyImageFile$Mean_Intensities
      colnames(AM) <- paste0("Mean", 1:input$bands)
      Med <- shinyImageFile$Median_Intensities
      colnames(Med) <- paste0("Median", 1:input$bands)
      if(input$thresh == "Otsu"){
        BG.method <- matrix(c("Otsu", NA, NA), nrow = 1,
                            ncol = 3, byrow = TRUE)
        colnames(BG.method) <- c("Background", "Offset", "Probability")
      }
      if(input$thresh == "Quantile"){
        BG.method <- matrix(c("quantile", NA, input$quantile1),
                            nrow = 1, ncol = 3, byrow = TRUE)
        colnames(BG.method) <- c("Background", "Offset", "Probability")
      }
      if(input$thresh == "Triangle"){
        BG.method <- matrix(c("triangle", input$tri_offset, NA), nrow = 1,
                            ncol = 3, byrow = TRUE)
        colnames(BG.method) <- c("Background", "Offset", "Probability")
      }
      if(input$thresh == "Li"){
        BG.method <- matrix(c("Li", NA, NA), nrow = 1,
                            ncol = 3, byrow = TRUE)
        colnames(BG.method) <- c("Background", "Offset", "Probability")
      }
      seg.list <- shinyImageFile$segmentation_list
      img <- seg.list[[1]][[1]]
      if(colorMode(img) > 0){
        MODE <- input$channel
        DF <- data.frame("File" = shinyImageFile$filename,
                         "Mode" = MODE,
                         "Strip" = input$selectStrip,
                         BG.method, AM, Med,
                         check.names = FALSE)
      }else{
        DF <- data.frame("File" = shinyImageFile$filename,
                         "Mode" = NA,
                         "Strip" = input$selectStrip,
                         BG.method, AM, Med,
                         check.names = FALSE)
      }
      if(inherits(try(IntensData, silent = TRUE), "try-error"))
        IntensData <<- DF
      else
        IntensData <<- rbind(IntensData, DF)

      output$intens <- renderDT({
        DF <- IntensData
        datatable(DF)
      })
      output$plot3 <- NULL
      output$plot4 <- NULL
      output$threshPlots <- NULL
      if(!is.null(shinyImageFile$Threshold))
        shinyImageFile$Threshold <- NULL
      if(!is.null(shinyImageFile$Mean_Intensities))
        shinyImageFile$Mean_Intensities <- NULL
      if(!is.null(shinyImageFile$Median_Intensities))
        shinyImageFile$Median_Intensities <- NULL
    })
  })

  output$intens <- renderDT({
    DF <- IntensData
    datatable(DF)
  })

  observe({recursiveShowIntensData()})
  recursiveShowIntensData <- eventReactive(input$showIntensData,{
    isolate({
      updateF7Tabs(session=session, id="tabs", selected = "IntensityData")
    })
  })

  observe({recursiveDelete()})
  recursiveDelete <- eventReactive(input$deleteData,{
    isolate({
      IntensData <<- NULL
      output$intens <- renderDT({})
    })
  })


  observe({recursiveRefresh()})
  recursiveRefresh <- eventReactive(input$refreshData,{
    isolate({
      output$intens <- renderDT({
        DF <- IntensData
        datatable(DF)
      })
    })
  })

  observe({recursiveUploadIntens()})
  recursiveUploadIntens <- eventReactive(input$intensFile,{
    isolate({
      req(input$intensFile)
      DF <- tryCatch({
        read.csv(input$intensFile$datapath, header = TRUE,
                 check.names = FALSE)
      }, error = function(e) {
        f7Toast(text="Error: Invalid intensity file", position="top", session=session)
        return()
      })
      if (is.null(DF)) return()
      IntensData <<- DF
      output$intens <- renderDT({
        datatable(DF)
      })
    })
  })

  observe({recursiveQuant()})

  recursiveQuant <- eventReactive(input$quant,{
    updateF7Tabs(session=session, id="tabs", selected = "Quantification")
  })


  #Download code
  output$downloadData <- downloadHandler(
    filename = "IntensityData.csv",
    content = function(file) {
      write.csv(IntensData, file, row.names = FALSE)
    }
  )

  # Quantification module ------------------------------------------------------
  observeEvent(input$quanData, {
    tryCatch({
      quanData <<- read.csv(input$quanData$datapath, header = TRUE,
                            check.names = TRUE)
    }, error = function(e) {
      f7Toast(text="Error: Invalid calibration data", position="top", session=session)
      return()
    })
  })

  observeEvent(input$model, {
    tryCatch({
      calFun <<- readRDS(input$model$datapath)
    }, error = function(e) {
      f7Toast(text="Error: Invalid model loaded", position="top", session=session)
      return()
    })
  })

  observe({predictConc()})

  predictConc <- eventReactive(input$predict, {
    isolate({
      if ((input$quanUpload == "Upload Data" & is.null(quanData)) | (input$quanUpload == "Use Intensity Data" & is.null(IntensData))) {
        f7Toast(text="Error: No valid intensity data found", position="top", session=session)
        output$quant <- renderDT({})
        return()
      }
      if (!is.null(calFun)) {
        if (input$quanUpload == "Upload Data") {
          calConc <- tryCatch({
            calFun(quanData)
          },
          error = function(e) {
            f7Toast(text="There was an issue fitting the model. Please try another model", position="top", session=session)
            return()
          })
          if (is.null(calConc)) return()
          predictData <<- cbind(quanData, calConc)
          output$quant <- renderDT({
            DF <- predictData
            datatable(DF)
          })
        } else if(input$quanUpload == "Use Intensity Data") {
          calConc <- tryCatch({
            calFun(IntensData)
          },
          error = function(e) {
            f7Toast(text="Error: There was an issue fitting the model. Please try another model", position="top", session=session)
            return()
          })
          if (is.null(calConc)) return()
          predictData <<- cbind(IntensData, calConc)
          output$quant <- renderDT({
            DF <- predictData
            datatable(DF)
          })
        } 
      } else {
        f7Toast(text="Error: No valid model loaded. Please try again", position="top", session=session)
        output$quant <- renderDT({})
      }
    })
  })

  #allows user to download prediction
  output$downloadData4 <- downloadHandler(
    filename = "PredictData.csv",
    content = function(file) {
      write.csv(predictData, file, row.names = FALSE)
    }
  )
}

core_ui <- function(request) {
  tagList(
    fluidPage(
      theme = shinytheme("sandstone"),
      useShinyjs(),
      titlePanel("LFA App core"),
      tags$style(type='text/css', "#stop { float:right; }"),
      actionButton("stop", "Quit App"),
      tabsetPanel(id = "tabs",
                  ## Start of Tab Image Editor
                  tabPanel("Cropping and Segmentation", value = "tab1",
                           sidebarLayout(
                             sidebarPanel(
                               radioButtons("upload",
                                            label = ("Upload Image or Choose Sample"),
                                            choices = list("Upload Image" = 1,
                                                           "Sample Image" = 2),
                                            selected = 1),
                               conditionalPanel(
                                 condition = "input.upload == 1",
                                 fileInput(inputId = 'file1',
                                           label = 'Upload Image',
                                           placeholder = 'JPEG, PNG, and TIFF are supported',
                                           accept = c(
                                             "image/jpeg",
                                             "image/x-png",
                                             "image/tiff",
                                             ".jpg",
                                             ".png",
                                             ".tiff"))
                               ),
                               uiOutput("rotatePanel"),
                               hr(style="border-color: black"),
                               h5("Set number of strips and number of lines per strip",
                                  style="font-weight:bold"),
                               sliderInput("strips", "Number of strips:",
                                           min = 1, max = 10, value = 1),
                               sliderInput("bands", "Number of lines:",
                                           min = 2, max = 6, value = 2),
                             ),
                             mainPanel(
                               h3('Cropping and Segmentation', align = "center"),
                               plotOutput("plot1",
                                          click = "plot_click",
                                          dblclick = "plot_dblclick",
                                          hover = "plot_hover",
                                          brush = "plot_brush"),
                               br(),
                               h6("Click and drag to select a region of interest. Double click on the selected region to zoom.", align = "center"),
                               br(),
                               column(6,
                                      actionButton("reset", label = "Reset"),
                                      tags$style(type='text/css', "#reset { display: block; width:70%; margin-left: auto; margin-right:auto;}"),
                               ),
                               column(6, shinyjs::disabled(
                                 actionButton("segmentation", label = "Apply Segmentation")),
                                 tags$style(type='text/css', "#segmentation { display: block; width:70%; margin-left: auto; margin-right:auto;}"),
                               )
                             )
                           )
                  ), # END OF TAB PANEL
                  ## Start of Tab Background Correction
                  tabPanel("Background", value = "tab2",
                           sidebarLayout(
                             sidebarPanel(
                               numericInput(inputId = "selectStrip",
                                            label = "Select strip:",
                                            value = 1,
                                            min = 1,
                                            max = 1,
                                            step = 1,
                                            width = NULL
                               ),
                               hr(style="border-color: black"),
                               h5("Select threshold method",
                                  style="font-weight:bold"),
                               radioButtons("colorImage",
                                            label = ("Color image?"),
                                            choices = list("No" = 1,
                                                           "Yes" = 2),
                                            selected = 1),
                               conditionalPanel(
                                 condition = "input.colorImage == 2",
                                 radioButtons("channel",
                                              label = ("Conversion mode"),
                                              choices = list("luminance",
                                                             "gray",
                                                             "red",
                                                             "green",
                                                             "blue"),
                                              selected = "luminance")
                               ),
                               radioButtons("invert",
                                            label = ("Lines are darker than background?"),
                                            choices = list("No" = FALSE,
                                                           "Yes" = TRUE),
                                            selected = FALSE),
                               radioButtons("thresh",
                                            label = ("Threshold method"),
                                            choices = list("Otsu" = 1,
                                                           "Quantile" = 2,
                                                           "Triangle" = 3,
                                                           "Li" = 4),
                                            selected = 1),
                               conditionalPanel(
                                 condition = "input.thresh == 3",
                                 numericInput(inputId = "tri_offset",
                                              label = "Offset:",
                                              value = 0.2,
                                              min = 0,
                                              max = 1,
                                              step = 0.01,
                                              width = NULL)
                               ),
                               conditionalPanel(
                                 condition = "input.thresh == 2",
                                 numericInput(inputId = "quantile1",
                                              label = "Probability [%]:",
                                              value = 99,
                                              min = 0,
                                              max = 100,
                                              step = 0.1,
                                              width = NULL
                                 )
                               ),
                               actionButton("threshold", label = "Apply Threshold"), br(),
                               hr(style="border-color: black"),
                               actionButton("data", label = "Add To Intensity Data"), br(),
                               hr(style="border-color: black"),
                               actionButton("showIntensData", label = "Switch To Intensity Data")
                             ),
                             mainPanel(
                               HTML(
                                 paste(
                                   h3('Background Correction', align = "center"),
                                   verbatimTextOutput("thresh"),br(),
                                   h4('Signal Intensity Above Background', align = "center"),
                                   plotOutput("plot3"),
                                   h4('Lines After Background Subtraction', align = "center"),
                                   plotOutput("plot4"),
                                   verbatimTextOutput("meanIntens"),
                                   verbatimTextOutput("medianIntens"),
                                   '<br/>','<br/>'
                                 )
                               ),
                               width = 8
                             )
                           )
                  ), # END OF TAB PANEL
                  ## Start of Tab Data
                  tabPanel("Intensity Data", value = "tab3",
                           sidebarLayout(
                             sidebarPanel(
                               h5("Download intensity data", style="font-weight:bold"),
                               downloadButton("downloadData", "Download Data"), br(),
                               hr(style="border-color: black"),
                               h5("For restart with new data", style="font-weight:bold"),
                               actionButton("deleteData", label = "Delete Data"), br(),
                             ),
                             mainPanel(
                               DTOutput("intens")
                             )
                           )
                  )
      ) # END OF TAB SET PANEL
    )
  )
}

core_mobile_ui <- f7Page(
  allowPWA=TRUE,
  options=list(dark=FALSE),
  tags$head(
    tags$script("
              $(document).ready(function() {
                var plot = document.getElementById('plot1')

                plot.addEventListener('touchmove', function (e) {
                  var touch = e.changedTouches[0];
                  var mouseEvent = new MouseEvent('mousemove', {
                    view: window,
                    bubbles: true,
                    cancelable: true,
                    screenX: touch.screenX,
                    screenY: touch.screenY,
                    clientX: touch.clientX,
                    clientY: touch.clientY
                  })
                  touch.target.dispatchEvent(mouseEvent);
                  e.preventDefault()
                }, { passive: false });

                plot.addEventListener('touchstart', function(e) {
                  var touch = e.changedTouches[0];
                  var mouseEvent = new MouseEvent('mousedown', {
                    view: window,
                    bubbles: true,
                    cancelable: true,
                    screenX: touch.screenX,
                    screenY: touch.screenY,
                    clientX: touch.clientX,
                    clientY: touch.clientY
                  })
                  touch.target.dispatchEvent(mouseEvent);
                  e.preventDefault()
                }, { passive: false });

                plot.addEventListener('touchstart', function(e) {
                  var touch = e.changedTouches[0];
                  var mouseEvent = new MouseEvent('click', {
                    view: window,
                    bubbles: true,
                    cancelable: true,
                    screenX: touch.screenX,
                    screenY: touch.screenY,
                    clientX: touch.clientX,
                    clientY: touch.clientY
                  })
                  touch.target.dispatchEvent(mouseEvent);
                  e.preventDefault()
                }, { passive: false });

                plot.addEventListener('touchend', function(e) {
                  var touch = e.changedTouches[0];
                  var mouseEvent = new MouseEvent('mouseup', {
                    view: window,
                    bubbles: true,
                    cancelable: true,
                    screenX: touch.screenX,
                    screenY: touch.screenY,
                    clientX: touch.clientX,
                    clientY: touch.clientY
                  })
                  touch.target.dispatchEvent(mouseEvent);
                  e.preventDefault()
                }, { passive: false });
              })
            "),
    tags$style("#plot1 { touch-action: none; }")
  ),
  title = "LFApp Mobile core",
  f7TabLayout(
    # Maybe the navbar will be removed later.
    navbar = f7Navbar(
      title="LFApp mobile core"
    ),
    # Each tab has it's own content
    f7Tabs(
      animated = TRUE,
      id = "tabs",
      f7Tab(
        title = "Crop & Segmentation",
        tabName = "CropSegmentation",
        icon = f7Icon("tray_arrow_up"),
        active = TRUE,
        # Upload file block
        f7Block(
          strong = TRUE,
          inset = FALSE,
          f7Radio(inputId= "upload", label="Upload image or choose sample", choices=list("Upload image", "Sample"), selected = "Sample"),
          conditionalPanel(
            condition = "input.upload == 'Upload image'",
            f7File(inputId = 'file1',
                   label = 'Upload Image',
                   placeholder = 'JPEG, PNG, and TIFF are supported',
                   accept = c(
                     "image/jpeg",
                     "image/x-png",
                     "image/tiff",
                     ".jpg",
                     ".png",
                     ".tiff"))
            )
        ),
        f7Accordion(
          f7AccordionItem(
            title = "Rotate Image",
            tagList(
              f7Slider("rotate", label="Angle",
                          min=-45, max=45, value=0, scale=TRUE),
              br(),
              f7Segment(
                f7Button(inputId="rotateCCW", label = "-90"),
                f7Button(inputId="rotateCW", label = "+90"),
                f7Button(inputId="fliphor", label = "FH"),
                f7Button(inputId="flipver", label = "FV")
              )
            )
          )
        ),
        f7Block(
          f7BlockTitle("Set number of strips and number of lines per strip"),
          strong = TRUE,
          inset = FALSE,
          f7Slider("strips", label="Number of strips", min=1, max=10, value=1, scale=TRUE, scaleSteps=9),
          f7Slider("bands", label="Number of lines", min=2, max=6, value=2, scale=TRUE, scaleSteps=4)
        ),
        f7Block(
          f7BlockTitle("Cropping and Segmentation", size="medium"),
          strong = TRUE,
          inset = FALSE,
          # The content of the tab goes below this
          plotOutput("plot1",
                     click = "plot_click",
                     dblclick = "plot_dblclick",
                     hover = hoverOpts("plot_hover", delay = 5000, clip = TRUE),
                     brush = "plot_brush"),
          h5("Click and drag to select a region of interest. Double click on the selected region to zoom.", align = "center"),
          uiOutput("cropButtons")
        )
      ),
      f7Tab(
        title = "Background",
        tabName = "Background",
        icon = f7Icon("circle_lefthalf_fill"),
        active = FALSE,
        f7Block(
          strong = TRUE,
          inset = FALSE,
          "Select strip: ",
          f7Stepper("selectStrip", label = "", min=1, max=1, value=1, size="small"),
          f7Accordion(
            f7AccordionItem(
              title = "Color image?",
              f7Radio("channel", label="Conversion mode", choices=list("luminance", "gray", "red", "green", "blue"), selected = "luminance"),
            ),
          ),
          f7Radio("invert", label="Lines darker than background?", choices=list("No", "Yes"), selected = "No"),
          f7Radio("thresh", label="Threshold", choices=list("Otsu", "Quantile", "Triangle", "Li"), selected = "Otsu"),
          conditionalPanel(
            condition = "input.thresh == 'Quantile'",
            f7Stepper(inputId = "quantile1",
                     label = "Probability [%]:",
                     value = 99,
                     min = 0,
                     max = 100,
                     step = 0.5,
                     manual = TRUE)
            ),
          conditionalPanel(
            condition = "input.thresh == 'Triangle'",
            f7Stepper(inputId = "tri_offset",
                     label = "Offset:",
                     value = 0.2,
                     min = 0,
                     max = 1,
                     step = 0.1,
                     manual = TRUE)
          ),
          f7Segment(
            f7Button(inputId="threshold", label = "Apply Threshold")
          )
        ),
        uiOutput("threshPlots"),
      ),
      f7Tab(
        title = "Intensity Data",
        tabName = "IntensityData",
        icon = f7Icon("table"),
        active = FALSE,
        f7Block(
          f7Accordion(
            f7AccordionItem(
              title = "Upload existing intensity data",
              f7File(inputId = 'intensFile',
                     label = 'Select CSV file',
                     accept = c("text/csv",
                                "text/comma-separated-values,text/plain",
                                ".csv"))
            )
        ),
        f7Block(
          strong = TRUE,
          f7Block(
            style = "overflow-x:scroll",
            DTOutput("intens")
          ),
          f7Segment(
            f7DownloadButton("downloadData", label = "Download Data"),
            f7Button("deleteData", color="red", label="Delete Data")
          )
          )
        )
      )
    )
  )
)

analysis_mobile_ui <- f7Page(
  allowPWA=TRUE,
  options=list(dark=FALSE),
  tags$head(
    tags$script("
              $(document).ready(function() {
                var plot = document.getElementById('plot1')

                plot.addEventListener('touchmove', function (e) {
                  var touch = e.changedTouches[0];
                  var mouseEvent = new MouseEvent('mousemove', {
                    view: window,
                    bubbles: true,
                    cancelable: true,
                    screenX: touch.screenX,
                    screenY: touch.screenY,
                    clientX: touch.clientX,
                    clientY: touch.clientY
                  })
                  touch.target.dispatchEvent(mouseEvent);
                  e.preventDefault()
                }, { passive: false });

                plot.addEventListener('touchstart', function(e) {
                  var touch = e.changedTouches[0];
                  var mouseEvent = new MouseEvent('mousedown', {
                    view: window,
                    bubbles: true,
                    cancelable: true,
                    screenX: touch.screenX,
                    screenY: touch.screenY,
                    clientX: touch.clientX,
                    clientY: touch.clientY
                  })
                  touch.target.dispatchEvent(mouseEvent);
                  e.preventDefault()
                }, { passive: false });

                plot.addEventListener('touchstart', function(e) {
                  var touch = e.changedTouches[0];
                  var mouseEvent = new MouseEvent('click', {
                    view: window,
                    bubbles: true,
                    cancelable: true,
                    screenX: touch.screenX,
                    screenY: touch.screenY,
                    clientX: touch.clientX,
                    clientY: touch.clientY
                  })
                  touch.target.dispatchEvent(mouseEvent);
                  e.preventDefault()
                }, { passive: false });

                plot.addEventListener('touchend', function(e) {
                  var touch = e.changedTouches[0];
                  var mouseEvent = new MouseEvent('mouseup', {
                    view: window,
                    bubbles: true,
                    cancelable: true,
                    screenX: touch.screenX,
                    screenY: touch.screenY,
                    clientX: touch.clientX,
                    clientY: touch.clientY
                  })
                  touch.target.dispatchEvent(mouseEvent);
                  e.preventDefault()
                }, { passive: false });
              })
            "),
    tags$style("#plot1 { touch-action: none; }")
  ),
  title = "LFA Mobile app",
  f7TabLayout(
    # Maybe the navbar will be removed later.
    navbar = f7Navbar(
      title="LFApp mobile analysis"
    ),
    # Each tab has it's own content
    f7Tabs(
      animated = TRUE,
      id = "tabs",
      f7Tab(
        title = "Crop & Segmentation",
        tabName = "CropSegmentation",
        icon = f7Icon("tray_arrow_up"),
        active = TRUE,
        # Upload file block
        f7Block(
          strong = TRUE,
          inset = FALSE,
          f7Radio(inputId= "upload", label="Upload image or choose sample", choices=list("Upload image", "Sample"), selected = "Sample"),
          conditionalPanel(
            condition = "input.upload == 'Upload image'",
            f7File(inputId = 'file1',
                   label = 'Upload Image',
                   placeholder = 'JPEG, PNG, and TIFF are supported',
                   accept = c(
                     "image/jpeg",
                     "image/x-png",
                     "image/tiff",
                     ".jpg",
                     ".png",
                     ".tiff"))
            )
        ),
        f7Accordion(
          f7AccordionItem(
            title = "Rotate Image",
            tagList(
              f7Slider("rotate", label="Angle",
                          min=-45, max=45, value=0, scale=TRUE),
              br(),
              f7Segment(
                f7Button(inputId="rotateCCW", label = "-90"),
                f7Button(inputId="rotateCW", label = "+90"),
                f7Button(inputId="fliphor", label = "FH"),
                f7Button(inputId="flipver", label = "FV")
              )
            )
          )
        ),
        f7Block(
          f7BlockTitle("Set number of strips and number of lines per strip"),
          strong = TRUE,
          inset = FALSE,
          f7Slider("strips", label="Number of strips", min=1, max=10, value=1, scale=TRUE, scaleSteps=9),
          f7Slider("bands", label="Number of lines", min=2, max=6, value=2, scale=TRUE, scaleSteps=4)
        ),
        f7Block(
          f7BlockTitle("Cropping and Segmentation", size="medium"),
          strong = TRUE,
          inset = FALSE,
          # The content of the tab goes below this
          plotOutput("plot1",
                     click = "plot_click",
                     dblclick = "plot_dblclick",
                     hover = hoverOpts("plot_hover", delay = 5000, clip = TRUE),
                     brush = "plot_brush"),
          h5("Click and drag to select a region of interest. Double click on the selected region to zoom.", align = "center"),
          uiOutput("cropButtons")
        )
      ),
      f7Tab(
        title = "Background",
        tabName = "Background",
        icon = f7Icon("circle_lefthalf_fill"),
        active = FALSE,
        f7Block(
          strong = TRUE,
          inset = FALSE,
          "Select strip: ",
          f7Stepper("selectStrip", label = "", min=1, max=1, value=1, size="small"),
          f7Accordion(
            f7AccordionItem(
              title = "Color image?",
              f7Radio("channel", label="Conversion mode", choices=list("luminance", "gray", "red", "green", "blue"), selected = "luminance"),
            ),
          ),
          f7Radio("invert", label="Lines darker than background?", choices=list("No", "Yes"), selected = "No"),
          f7Radio("thresh", label="Threshold", choices=list("Otsu", "Quantile", "Triangle", "Li"), selected = "Otsu"),
          conditionalPanel(
            condition = "input.thresh == 'Quantile'",
            f7Stepper(inputId = "quantile1",
                     label = "Probability [%]:",
                     value = 99,
                     min = 0,
                     max = 100,
                     step = 0.5,
                     manual = TRUE)
            ),
          conditionalPanel(
            condition = "input.thresh == 'Triangle'",
            f7Stepper(inputId = "tri_offset",
                     label = "Offset:",
                     value = 0.2,
                     min = 0,
                     max = 1,
                     step = 0.1,
                     manual = TRUE)
          ),
          f7Segment(
            f7Button(inputId="threshold", label = "Apply Threshold")
          )
        ),
        uiOutput("threshPlots"),
      ),
      f7Tab(
        title = "Intensity Data",
        tabName = "IntensityData",
        icon = f7Icon("table"),
        active = FALSE,
        f7Block(
          f7Accordion(
            f7AccordionItem(
              title = "Upload existing intensity data",
              f7File(inputId = 'intensFile',
                     label = 'Select CSV file',
                     accept = c("text/csv",
                                "text/comma-separated-values,text/plain",
                                ".csv"))
            )
        ),
        f7Block(
          strong = TRUE,
          f7Block(
            style = "overflow-x:scroll",
            DTOutput("intens")
          ),
          f7Segment(
            f7DownloadButton("downloadData", label = "Download Data"),
            f7Button("deleteData", color="red", label="Delete Data")
          ),
          f7Button(inputId="expInfo", label="Switch to experiment info")
          )
        )
      ),
      f7Tab(
        title = "Experiment Info",
        tabName = "ExperimentInfo",
        icon = f7Icon("info_circle"),
        active = FALSE,
        f7Block(
          f7Accordion(
            f7AccordionItem(
              title = "Upload experiment info or upload existing merged data",
              f7Accordion(
                f7File(inputId = 'expFile',
                       label = 'Select CSV file',
                       accept = c("text/csv",
                                  "text/comma-separated-values,text/plain",
                                  ".csv")),
                f7AccordionItem(
                  title = "CSV options",
                  f7Checkbox("header", "Header", TRUE),
                  f7Radio("sep", "Separator",
                          choices = c("Comma ( , )",
                                      "Semicolon ( ; )",
                                      "Tab ( \\t )"),
                          selected = "Comma ( , )"),
                  f7Radio("quote", "Quote",
                          choices = c("None",
                                      'Double Quote ( " )',
                                      "Single Quote ( ' )"),
                          selected = "None"))
                )
              ),
            f7AccordionItem(
              title = "Merge datasets",
              "Select ID columns and merge datasets",
              f7Text("mergeIntens", label = "ID Column Intensity Data", value = "File"),
              f7Text("mergeExp", label = "ID Column Experiment Info", value = "File"),
              f7Segment(
                f7Button("merge", label = "Merge With Intensity Data")
              )
            ),
          ),
          f7Block(
            strong = TRUE,
            f7Block(
              strong = TRUE,
              style = "overflow-x:scroll",
              DTOutput("experiment")
            ),
            f7Segment(
              f7DownloadButton("downloadData2", label = "Download Data"),
              f7Button("deleteData2", color="red", label="Delete Data")

            ),
            f7Block(
              f7Button("prepare", label = "Prepare Calibration")
            )
          )
        )
      ),
      f7Tab(
        title = "Calibration",
        tabName = "Calibration",
        icon = f7Icon("graph_square"),
        active = FALSE,
        f7Block(
          f7Accordion(
            f7AccordionItem(
              title = "Upload existing data for calibration",
              f7File(inputId = 'prepFile',
                     label = 'Select CSV file',
                     accept = c("text/csv",
                                "text/comma-separated-values,text/plain",
                                ".csv"))
            ),
            f7AccordionItem(
              title = "Furter preprocessing steps",
              f7Radio("radioPrepro",
                      label = "",
                      choices = c("None",
                                  "Average technical replicates",
                                  "Reshape from long to wide"),
                      selected = "None"),
              conditionalPanel(
                condition = "input.radioPrepro == 'Average technical replicates'",
                f7Text("combRepsColSI", label = "Column with sample information:", value = "Sample"),
                f7Stepper(inputId = "colorsBands",
                          label = "Number of analytes/colors per line:",
                          value = 1,
                          min = 1,
                          max = 5,
                          step = 1,
                ),
                conditionalPanel(
                  condition = "input.colorsBands > 1",
                  f7Text("combRepsColCL", label = "Column with color information:", value = "Color"),
                ),
                f7Radio("radioReps",
                        label = ("Choose measure for averaging:"),
                        choices = list("Mean",
                                       "Median"),
                        selected = "Mean"),
                f7Segment(
                  f7Button("combReps", label = "Average Technical Replicates")
                )
              ),
              conditionalPanel(
                condition = "input.radioPrepro == 'Reshape from long to wide'",
                f7Text("combRepsColSI2", label = "Column with sample information:", value = "Sample"),
                f7Text("reshapeCol", label = "Reshape attribute column:", value = "Color"),
                f7Segment(
                  f7Button("reshapeWide", label = "Reshape")
                )
              )
            ),
            f7AccordionItem(
              title = "Calibration",
              open = TRUE,
              f7Text("analysisName","Analysis name:", value="Model1"),
              f7Radio("chosenModel",
                      label = "Choose model",
                      choices = list("Linear model (lm)",
                                     "Local polynomial model (loess)",
                                     "Generalized additive model (gam)"),
                      selected = "Linear model (lm)"),
              f7Text("concVar", label = "Specify column with concentration",
                       placeholder = "Enter column with concentration"),
              f7TextArea("respVar", label = "Specify response variable (R expresssion)",
                         placeholder = "e.g. Mean2 / (Mean1 + Mean2)"),
              f7TextArea("subset", label = "Optional: specify subset (logical R expression)",
                         placeholder = "e.g. Background == 'Otsu'"),
              f7Checkbox("useLog", "Logarithmize concentration", value=FALSE),
              f7Block(
                strong = TRUE,
                f7Block(
                  strong = TRUE,
                  style = "overflow-x:scroll",
                  verbatimTextOutput("folder"),
                  DTOutput("calibration")
                ),
                f7Segment(
                  f7DownloadButton("downloadData3", label = "Download Data"),
                  f7Button("deleteData3", color="red", label="Delete Data")
                )
              ),
              f7Segment(
                f7Text("folder","Working directory:", value=file.path(fs::path_home(), "Documents/LFApp"),
                       placeholder=file.path(fs::path_home(), "Documents/LFApp")),
                f7Button("runCali", label = "Run Calibration Analysis")
              )
            )
          ),
        )
      ),
      f7Tab(
        title = "Results",
        tabName = "Results",
        icon = f7Icon("doc_text_search"),
        active = FALSE,
        f7Block(
          strong = TRUE,
          h3("Open analysis report"),
          f7Button("openReport", label = "Open")
        ),
        uiOutput("results")
      ),
      f7Tab(
        title = "Quantification",
        tabName = "Quantification",
        icon = f7Icon("gauge"),
        active = FALSE,
        f7Block(
          strong = TRUE,
          inset = FALSE,
          f7Radio(inputId= "quanUpload",
                  label="You can use Intensity Data or upload new data",
                  choices=list("Use Intensity Data", "Upload Data"),
                  selected = "Upload Data"),
          conditionalPanel(
            condition = "input.quanUpload == 'Upload Data'",
            f7File(inputId = 'quanData',
                   label = 'Upload Data',
                   accept = c("csv"))
          )
        ),
        f7Block(
          strong = TRUE,
          h3("Load existing model from file"),
          f7File(inputId = 'model',
                 label = 'Select model',
                 accept = ".rds",
                 placeholder = "RDS file"),
          h3("Predict concentration"),
          f7Button("predict", label = "Predict"),
          h3("Download concentration data"),
          f7DownloadButton("downloadData4", "Download data")
        ),
        f7Block(
          strong = TRUE,
          style = "overflow-x:scroll",
          DTOutput("quant")
        )
      )
    )
  )
)

threshold_li <- function(image, tolerance=NULL, initial_guess=NULL, iter_callback=NULL) {
  # For Li's algorithm to work, the image should be positive
  image_min <- min(image)
  image <- image - image_min
  # Tolerance has to be positive or there is a risk of while loop to be infinite.
  if (is.null(tolerance)) {
    tolerance <- abs(min(diff(as.vector(image))) / 2)
  }
  # Initial estimate for iterations
  if (is.null(initial_guess)) {
    t_next <- mean(image)
  } else if (is(initial_guess,"function")){
    t_next <- initial_guess(image)
  } else if (is.numeric(initial_guess)) {
    t_next <- initial_guess - image_min
    image_max <- max(image) + image_min
    if (!t_next>0 & !t_next<max(image)) {
      stop(sprintf("The threshold_li must be greater than 0 and lesser than max
                   value of the image. threshold_li is %s", threshold_li))
    } else {
      stop("The initial_guess has incorrect class. It should be numeric or a 
           function that returns numeric value.")
    }
  }
  # The difference between t_next and t_curr should be equal to the tolerance.
  t_curr <- tolerance * (-2)
  
  if (!is.null(iter_callback)) {
    iter_callback(t_next + image_min)
  } 
  
  # Stop the iteration when the difference between the new and old threshold
  # value is less than the tolerance
  while(abs(t_next - t_curr) > tolerance) {
    t_curr <- t_next
    foreground <- (image > t_curr)
    mean_fore <- mean(image[foreground])
    mean_back <- mean(image[!foreground])

    t_next <- ((mean_back - mean_fore) /
               (log(mean_back) - log(mean_fore)))

    if (!is.null(iter_callback)) {
      iter_callback(t_next + image_min)
    }
  }
  threshold <- t_next + image_min
  threshold
}

cal_server <- function( input, output, session ) {
  ###### FIRST TAB
  oldpar <- par(no.readonly = TRUE)
  on.exit(par(oldpar))
  
  oldopt <- options()
  on.exit(options(oldopt))
  options(shiny.maxRequestSize=100*1024^2) #file can be up to 50 mb; default is 5 mb
  ## initializations
  shinyImageFile <- reactiveValues(shiny_img_origin = NULL, shiny_img_cropped = NULL,
                                   shiny_img_final = NULL, Threshold = NULL)
  IntensData <- NULL
  ExpInfo <- NULL
  MergedData <- NULL
  FILENAME <- NULL
  fit <- NULL
  modelPlot <- NULL
  LOB <- NULL
  LOD <- NULL
  LOQ <- NULL
  predFunc <- NULL
  CalibrationData <- NULL
  
  startAutosave <- reactiveVal(value=FALSE)
  
  #checks upload for file input
  observe({
    #default: upload image
    if(input$upload == 1){
      output$plot1 <- renderPlot({
        if(is.null(input$file1)) {
          output$rotatePanel <- renderUI({})
        }
        validate(need(!is.null(input$file1), "Must upload a valid jpg, png, or tiff"))
      })
    }
    if(input$upload == 2){
      # using sample image
      img <- readImage(system.file("images", "sample.TIF", package="LFApp"))
      shinyImageFile$shiny_img_origin <- img
      shinyImageFile$shiny_img_cropped <- img
      shinyImageFile$shiny_img_final <- img
      
      shinyImageFile$filename <- "sample.TIF"
      #outputs image to plot1 -- main plot
      output$plot1 <- renderPlot({ EBImage::display(shinyImageFile$shiny_img_final, method = "raster") })
      drawRotatePanel()
    }
  }) # end of observe
  
  # NOTE renameUpload is completely unecessary. The app works without it perfectly.
  
  # #the datapath is different from the one needed to properly recognize photo
  # #so this function renames the file
  # renameUpload <- function(inFile){
  #   if(is.null(inFile))
  #     return(NULL)
  # 
  #   oldNames <- inFile$datapath
  #   newNames <- file.path(dirname(inFile$datapath), inFile$name)
  #   file.rename(from = oldNames, to = newNames)
  #   inFile$datapath <- newNames
  # 
  #   return(inFile$datapath)
  # }
  
  #if they enter a new file, their file will become the new imageFile
  observeEvent(input$file1, {
    shinyImageFile$filename <- input$file1$name
    # img <- readImage(renameUpload(input$file1)) # Commented it, because I think its redundant.
    img <- readImage(input$file1$datapath)
    shinyImageFile$shiny_img_origin <- img
    shinyImageFile$shiny_img_cropped <- img
    shinyImageFile$shiny_img_final <- img
    output$plot1 <- renderPlot({EBImage::display(img, method = "raster")})
    drawRotatePanel()
  })
  
  
  # NOTE This function draws rotation panel.
  drawRotatePanel <- function() {
    output$rotatePanel <- renderUI({
      tagList(
        sliderInput("rotate", "Rotate image",
                    min=-45, max=45, value=0),
        actionButton("rotateCCW", "-90"),
        actionButton("rotateCW", "+90"),
        actionButton("fliphor", "FH"),
        actionButton("flipver", "FV"),
      )
    })
  }
  
  
  observe({reactiveRotation()})
  
  reactiveRotation <- eventReactive(input$rotate, {
    isolate({
      if (!is.null(shinyImageFile$shiny_img_cropped)) {
        shinyImageFile$shiny_img_final <- EBImage::rotate(shinyImageFile$shiny_img_cropped, input$rotate,
                                                          bg.col="white")
        output$plot1 <- renderPlot({EBImage::display(shinyImageFile$shiny_img_final, method = "raster")})
        session$resetBrush("plot_brush")
      }
    })
  })
  
  observe({reactiveRotationCCW()})
  
  reactiveRotationCCW <- eventReactive(input$rotateCCW, {
    isolate({
      if (!is.null(shinyImageFile$shiny_img_cropped)) {
        shinyImageFile$shiny_img_cropped <- EBImage::rotate(shinyImageFile$shiny_img_cropped, -90)
        shinyImageFile$shiny_img_final <- shinyImageFile$shiny_img_cropped
        output$plot1 <- renderPlot({EBImage::display(shinyImageFile$shiny_img_final, method = "raster")})
        session$resetBrush("plot_brush")
      }
    })
  })
  
  observe({reactiveRotationCW()})
  
  reactiveRotationCW <- eventReactive(input$rotateCW, {
    isolate({
      if (!is.null(shinyImageFile$shiny_img_cropped)) {
        shinyImageFile$shiny_img_cropped <- EBImage::rotate(shinyImageFile$shiny_img_cropped, 90)
        shinyImageFile$shiny_img_final <- shinyImageFile$shiny_img_cropped
        output$plot1 <- renderPlot({EBImage::display(shinyImageFile$shiny_img_final, method = "raster")})
        session$resetBrush("plot_brush")
      }
    })
  })
  
  observe({reactiveRotationFlip()})
  
  reactiveRotationFlip <- eventReactive(input$fliphor, {
    isolate({
      if (!is.null(shinyImageFile$shiny_img_cropped)) {
        shinyImageFile$shiny_img_cropped <- EBImage::flip(shinyImageFile$shiny_img_cropped)
        shinyImageFile$shiny_img_final <- shinyImageFile$shiny_img_cropped
        output$plot1 <- renderPlot({EBImage::display(shinyImageFile$shiny_img_final, method = "raster")})
        session$resetBrush("plot_brush")
      }
    })
  })
  
  observe({reactiveRotationFlop()})
  
  reactiveRotationFlop <- eventReactive(input$flipver, {
    isolate({
      if (!is.null(shinyImageFile$shiny_img_cropped)) {
        shinyImageFile$shiny_img_cropped <- EBImage::flop(shinyImageFile$shiny_img_cropped)
        shinyImageFile$shiny_img_final <- shinyImageFile$shiny_img_cropped
        output$plot1 <- renderPlot({EBImage::display(shinyImageFile$shiny_img_final, method = "raster")})
        session$resetBrush("plot_brush")
      }
    })
  })
  
  croppedImage <- function(image, xmin, ymin, xmax, ymax){
    if(length(dim(image)) == 2)
      image <- image[xmin:xmax, ymin:ymax, drop = FALSE]
    else if(length(dim(image)) == 3)
      image <- image[xmin:xmax, ymin:ymax, ,drop = FALSE]
    return(image)
  }
  
  observe({resetImage()})
  
  resetImage <- eventReactive(input$reset,{
    isolate({
      # For now, resetting only the grid is enough. If there is a need of resseting the image
      # and rotation settings, uncommennt following three lines.
      shinyImageFile$shiny_img_cropped <- shinyImageFile$shiny_img_origin
      shinyImageFile$shiny_img_final <- shinyImageFile$shiny_img_cropped
      output$plot1 <- renderPlot({EBImage::display(shinyImageFile$shiny_img_final, method = "raster")})
      session$resetBrush("plot_brush")
      updateSliderInput(session, "rotate", value=0)
      shinyjs::disable("segmentation")
    })
  })
  
  
  #prompts shiny to look at recursive crop
  observe({recursiveCrop()})
  
  #only executes when keep is clicked
  recursiveCrop <- eventReactive(input$plot_dblclick,{
    isolate({
      p <- input$plot_brush
      validate(need(p$xmax <= dim(shinyImageFile$shiny_img_cropped)[1], 
                    "Highlighted portion is out of bounds on the x-axis"))
      validate(need(p$ymax <= dim(shinyImageFile$shiny_img_cropped)[2], 
                    "Highlighted portion is out of bounds on the y-axis"))
      validate(need(p$xmin >= 0, 
                    "Highlighted portion is out of bounds on the x-axis"))
      validate(need(p$ymin >= 0, 
                    "Highlighted portion is out of bounds on the y-axis"))
      shinyImageFile$shiny_img_cropped <- croppedImage(shinyImageFile$shiny_img_final, p$xmin, p$ymin, p$xmax, p$ymax)
      shinyImageFile$shiny_img_final <- shinyImageFile$shiny_img_cropped
      output$plot1 <- renderPlot({
        EBImage::display(shinyImageFile$shiny_img_final, method = "raster")
      })
      updateSliderInput(session, "rotate", value=0)
      session$resetBrush("plot_brush")
      shinyjs::enable("reset")
    })
    session$resetBrush("plot_brush")
    shinyjs::disable("segmentation")
  })
  
  observe({recursiveGrid()})
  
  recursiveGrid <- eventReactive(input$plot_brush,{
    isolate({
      p <- input$plot_brush
      output$plot1 <- renderPlot({
        EBImage::display(shinyImageFile$shiny_img_final, method = "raster")
        
        colcuts <- seq(p$xmin, p$xmax, length.out = input$strips + 1)
        rowcuts <- seq(p$ymin, p$ymax, length.out = 2*input$bands) # bands + spaces between bands
        
        for (x in colcuts) {
          lines(x = rep(x, 2), y = c(p$ymin, p$ymax), col="red")
        }
        for (y in rowcuts) {
          lines(x = c(p$xmin, p$xmax), y = rep(y, 2), col="red")
        }
      })
      shinyjs::enable("reset")
      shinyjs::enable("segmentation")
    })
  })
  
  observe({recursiveSegmentation()})
  
  #only executes when Apply Segmentation is clicked
  recursiveSegmentation <- eventReactive(input$segmentation,{
    isolate({
      p <- input$plot_brush
      # Check if the region of interest is out of the bounds
      if (p$xmax <= dim(shinyImageFile$shiny_img_cropped)[1] &&
          p$ymax <= dim(shinyImageFile$shiny_img_cropped)[2] &&
          p$xmin >= 0 &&
          p$ymin >= 0) {
        MAX <- dim(shinyImageFile$shiny_img_cropped)[1:2]
        colcuts <- seq(p$xmin, p$xmax, length.out = input$strips + 1)
        rowcuts <- seq(p$ymin, p$ymax, length.out = 2*input$bands)
        
        segmentation.list <- vector("list", length = input$strips)
        count <- 0
        for(i in 1:input$strips){
          tmp.list <- vector("list", length = 2*input$bands-1)
          for(j in 1:(2*input$bands-1)){
            img <- shinyImageFile$shiny_img_final
            if(length(dim(img)) == 2)
              img <- img[colcuts[i]:colcuts[i+1], rowcuts[j]:rowcuts[j+1]]
            else if(length(dim(img)) == 3)
              img <- img[colcuts[i]:colcuts[i+1], rowcuts[j]:rowcuts[j+1], , drop = FALSE]
            tmp.list[[j]] <- img
          }
          segmentation.list[[i]] <- tmp.list
        }
        shinyImageFile$cropping_grid <- list("columns" = colcuts, "rows" = rowcuts)
        shinyImageFile$segmentation_list <- segmentation.list
        updateTabsetPanel(session, "tabs", selected = "tab2")
      } else {
        showNotification("Error: The grid is out of bounds", duration = 5, type="error")
      }
    })
  })
  
  ################# END OF THE FIRST TAB
  
  ############### SECOND TAB
  
  observe({
    input$thresh
    updateNumericInput(session, "selectStrip", max=input$strips)
  })
  
  observe({input$channel})
  
  observe({recursiveThreshold()})
  
  recursiveThreshold <- eventReactive(input$threshold,{
    isolate({
      seg.list <- shinyImageFile$segmentation_list
      i <- input$selectStrip
      if(input$thresh == 2){
        Background <- vector(mode = "list", length = input$bands)
        for(j in 1:input$bands){
          img <- seg.list[[i]][[j]]
          if(colorMode(img) > 0){
            img <- 1-EBImage::channel(img, input$channel)
          }
          if(input$invert) {
            img <- 1 - img
          }
          Background[[j]] <- as.numeric(EBImage::imageData(img))
        }
        Background.Threshold <- quantile(unlist(Background),
                                         probs = input$quantile1/100)
        shinyImageFile$Threshold <- Background.Threshold
        output$plot3 <- renderPlot({
          par(mfcol = c(1, input$bands))
          Bands <- seq(1, 2*input$bands-1, by = 2)
          count <- 0
          for(j in Bands){
            count <- count + 1
            img <- seg.list[[i]][[j]]
            if(colorMode(img) > 0){
              img <- 1-EBImage::channel(img, input$channel)
            }
            if(input$invert) {
              img <- 1 - img
            }
            signal <- EBImage::imageData(img) > Background.Threshold
            EBImage::imageData(img) <- signal
            plot(img)
            title(paste0("Line ", count))
          }
        })
        shinyImageFile$Mean_Intensities <- matrix(0, nrow = 1, ncol = input$bands)
        shinyImageFile$Median_Intensities <- matrix(0, nrow = 1, ncol = input$bands)
        output$plot4 <- renderPlot({
          par(mfcol = c(1, input$bands))
          count <- 0
          Bands <- seq(1, 2*input$bands-1, by = 2)
          count <- 0
          for(j in Bands){
            count <- count + 1
            img <- seg.list[[i]][[j]]
            if(colorMode(img) > 0){
              img <- 1-EBImage::channel(img, input$channel)
            }
            if(input$invert) {
              img <- 1 - img
            }
            signal <- EBImage::imageData(img) > Background.Threshold
            EBImage::imageData(img) <- (EBImage::imageData(img) - Background.Threshold)*signal
            shinyImageFile$Mean_Intensities[1,count] <- mean(EBImage::imageData(img)[signal])
            shinyImageFile$Median_Intensities[1,count] <- median(EBImage::imageData(img)[signal])
            plot(img)
            title(paste0("Line ", count))
          }
        })
      }
      else if(input$thresh == 1){
        Background.Threshold <- numeric(input$bands)
        output$plot3 <- renderPlot({
          par(mfcol = c(1, input$bands))
          count1 <- 0
          Bands <- seq(1, 2*input$bands-1, by = 2)
          count2 <- 0
          for(j in Bands){
            count1 <- count1 + 1
            count2 <- count2 + 1
            img <- seg.list[[i]][[j]]
            if(colorMode(img) > 0){
              img <- 1-EBImage::channel(img, input$channel)
            }
            if(input$invert) {
              img <- 1 - img
            }
            Background.Threshold[count1] <- otsu(img)
            signal <- EBImage::imageData(img) > Background.Threshold[count1]
            EBImage::imageData(img) <- signal
            plot(img)
            title(paste0("Line ", count2))
          }
          shinyImageFile$Threshold <- Background.Threshold
        })
        shinyImageFile$Mean_Intensities <- matrix(0, nrow = 1, ncol = input$bands)
        shinyImageFile$Median_Intensities <- matrix(0, nrow = 1, ncol = input$bands)
        output$plot4 <- renderPlot({
          par(mfcol = c(1, input$bands))
          count1 <- 0
          Bands <- seq(1, 2*input$bands-1, by = 2)
          count2 <- 0
          for(j in Bands){
            count1 <- count1 + 1
            count2 <- count2 + 1
            img <- seg.list[[i]][[j]]
            if(colorMode(img) > 0){
              img <- 1-EBImage::channel(img, input$channel)
            }
            if(input$invert) {
              img <- 1 - img
            }
            thr <- otsu(img)
            signal <- EBImage::imageData(img) > thr
            EBImage::imageData(img) <- (EBImage::imageData(img) - thr)*signal
            shinyImageFile$Mean_Intensities[1,count1] <- mean(EBImage::imageData(img)[signal])
            shinyImageFile$Median_Intensities[1,count1] <- median(EBImage::imageData(img)[signal])
            plot(img)
            title(paste0("Line ", count2))
          }
        })
      }
      else if(input$thresh == 3){
        Background.Threshold <- numeric(input$bands)
        output$plot3 <- renderPlot({
          par(mfcol = c(1, input$bands))
          count1 <- 0
          Bands <- seq(1, 2*input$bands-1, by = 2)
          count2 <- 0
          for(j in Bands){
            count1 <- count1 + 1
            count2 <- count2 + 1
            img <- seg.list[[i]][[j]]
            if(colorMode(img) > 0){
              img <- 1-EBImage::channel(img, input$channel)
            }
            if(input$invert) {
              img <- 1 - img
            }
            Background.Threshold[count1] <- triangle(img, input$tri_offset)
            signal <- EBImage::imageData(img) > Background.Threshold[count1]
            EBImage::imageData(img) <- signal
            plot(img)
            title(paste0("Line ", count2))
          }
          shinyImageFile$Threshold <- Background.Threshold
        })
        shinyImageFile$Mean_Intensities <- matrix(0, nrow = 1, ncol = input$bands)
        shinyImageFile$Median_Intensities <- matrix(0, nrow = 1, ncol = input$bands)
        output$plot4 <- renderPlot({
          par(mfcol = c(1, input$bands))
          count1 <- 0
          Bands <- seq(1, 2*input$bands-1, by = 2)
          count2 <- 0
          for(j in Bands){
            count1 <- count1 + 1
            count2 <- count2 + 1
            img <- seg.list[[i]][[j]]
            if(colorMode(img) > 0){
              img <- 1-EBImage::channel(img, input$channel)
            }
            if(input$invert) {
              img <- 1 - img
            }
            thr <- triangle(img, input$tri_offset)
            signal <- EBImage::imageData(img) > thr
            EBImage::imageData(img) <- (EBImage::imageData(img) - thr)*signal
            shinyImageFile$Mean_Intensities[1,count1] <- mean(EBImage::imageData(img)[signal])
            shinyImageFile$Median_Intensities[1,count1] <- median(EBImage::imageData(img)[signal])
            plot(img)
            title(paste0("Line ", count2))
          }
        })
      }
      else if(input$thresh == 4){
        Background.Threshold <- numeric(input$bands)
        output$plot3 <- renderPlot({
          par(mfcol = c(1, input$bands))
          count1 <- 0
          Bands <- seq(1, 2*input$bands-1, by = 2)
          count2 <- 0
          for(j in Bands){
            count1 <- count1 + 1
            count2 <- count2 + 1
            img <- seg.list[[i]][[j]]
            if(colorMode(img) > 0){
              img <- 1-EBImage::channel(img, input$channel)
            }
            if(input$invert) {
              img <- 1 - img
            }
            
            Background.Threshold[count1] <- threshold_li(img)
            signal <- EBImage::imageData(img) > Background.Threshold[count1]
            EBImage::imageData(img) <- signal
            plot(img)
            title(paste0("Line ", count2))
          }
          shinyImageFile$Threshold <- Background.Threshold
        })
        shinyImageFile$Mean_Intensities <- matrix(0, nrow = 1, ncol = input$bands)
        shinyImageFile$Median_Intensities <- matrix(0, nrow = 1, ncol = input$bands)
        output$plot4 <- renderPlot({
          par(mfcol = c(1, input$bands))
          count1 <- 0
          Bands <- seq(1, 2*input$bands-1, by = 2)
          count2 <- 0
          for(j in Bands){
            count1 <- count1 + 1
            count2 <- count2 + 1
            img <- seg.list[[i]][[j]]
            if(colorMode(img) > 0){
              img <- 1-EBImage::channel(img, input$channel)
            }
            if(input$invert) {
              img <- 1 - img
            }
            thr <- threshold_li(img)
            signal <- EBImage::imageData(img) > thr
            EBImage::imageData(img) <- (EBImage::imageData(img) - thr)*signal
            shinyImageFile$Mean_Intensities[1,count1] <- mean(EBImage::imageData(img)[signal])
            shinyImageFile$Median_Intensities[1,count1] <- median(EBImage::imageData(img)[signal])
            plot(img)
            title(paste0("Line ", count2))
          }
        })
      }
    })
  })
  
  observe({recursiveData()})
  
  recursiveData <- eventReactive(input$data,{
    isolate({
      if (!is.null(shinyImageFile$Threshold)) {
        AM <- shinyImageFile$Mean_Intensities
        colnames(AM) <- paste0("Mean", 1:input$bands)
        Med <- shinyImageFile$Median_Intensities
        colnames(Med) <- paste0("Median", 1:input$bands)
        if(input$thresh == 1){
          BG.method <- matrix(c("Otsu", NA, NA), nrow = 1,
                              ncol = 3, byrow = TRUE)
          colnames(BG.method) <- c("Background", "Offset", "Probability")
        }
        if(input$thresh == 2){
          BG.method <- matrix(c("quantile", NA, input$quantile1),
                              nrow = 1, ncol = 3, byrow = TRUE)
          colnames(BG.method) <- c("Background", "Offset", "Probability")
        }
        if(input$thresh == 3){
          BG.method <- matrix(c("triangle", input$tri_offset, NA), nrow = 1, 
                              ncol = 3, byrow = TRUE)
          colnames(BG.method) <- c("Background", "Offset", "Probability")        
        }
        if(input$thresh == 4){
          BG.method <- matrix(c("Li", NA, NA), nrow = 1, 
                              ncol = 3, byrow = TRUE)
          colnames(BG.method) <- c("Background", "Offset", "Probability")        
        }
        seg.list <- shinyImageFile$segmentation_list
        img <- seg.list[[1]][[1]]
        if(colorMode(img) > 0){
          MODE <- input$channel
          DF <- data.frame("File" = shinyImageFile$filename,
                           "Mode" = MODE,
                           "Strip" = input$selectStrip,
                           BG.method, AM, Med,
                           check.names = TRUE)
        }else{
          DF <- data.frame("File" = shinyImageFile$filename,
                           "Mode" = NA,
                           "Strip" = input$selectStrip,
                           BG.method, AM, Med,
                           check.names = TRUE)
        }
        if(inherits(try(IntensData, silent = TRUE), "try-error"))
          IntensData <<- DF
        else
          IntensData <<- rbind(IntensData, DF)
        
        output$intens <- renderDT({
          DF <- IntensData
          datatable(DF,
                    options = list(
                      scrollX = TRUE
                    ))
        })
        output$plot3 <- NULL
        output$plot4 <- NULL
        if(!is.null(shinyImageFile$Threshold))
          shinyImageFile$Threshold <- NULL
        if(!is.null(shinyImageFile$Mean_Intensities))
          shinyImageFile$Mean_Intensities <- NULL
        if(!is.null(shinyImageFile$Median_Intensities))
          shinyImageFile$Median_Intensities <- NULL
        
        # Save the workspace every time you add to intensity data
        # save(shinyImageFile, IntensData, 
        #      ExpInfo, MergedData, fit, 
        #      modelPlot, LOB, LOD, LOQ,
        #      file=file.path(fs::path_home(), "Documents/LFApp/cal_autosave.RData"))
        # showNotification("Workspace saved", duration=2, type="message")
      }
    })
  })
  
  observe({recursiveShowIntensData()})
  recursiveShowIntensData <- eventReactive(input$showIntensData,{
    isolate({
      updateTabsetPanel(session, "tabs", selected = "tab3")
    })
  })
  
  observe({recursiveDelete()})
  recursiveDelete <- eventReactive(input$deleteData,{
    isolate({
      IntensData <<- NULL
      output$intens <- renderDT({})
    })
  })
  
  observe({recursiveDelete2()})
  recursiveDelete2 <- eventReactive(input$deleteData2,{
    isolate({
      ExpInfo <<- NULL
      MergedData <<- NULL
      output$experiment <- renderDT({})
    })
  })
  
  observe({recursiveDelete3()})
  recursiveDelete3 <- eventReactive(input$deleteData3,{
    isolate({
      MergedData <<- NULL
      CalibrationData <<- NULL
      output$calibration <- renderDT({})
    })
  })
  
  observe({recursiveRefresh()})
  recursiveRefresh <- eventReactive(input$refreshData,{
    isolate({
      output$intens <- renderDT({
        DF <- IntensData
        datatable(DF,
                  options = list(
                    scrollX = TRUE
                  ))
      })
    })
  })
  
  observe({recursiveRefresh2()})
  recursiveRefresh2 <- eventReactive(input$refreshData2,{
    isolate({
      output$experiment <- renderDT({
        DF <- MergedData
        datatable(DF,
                  options = list(
                    scrollX = TRUE
                  ))
      })
    })
  })
  
  observe({recursiveRefresh3()})
  recursiveRefresh3 <- eventReactive(input$refreshData3,{
    isolate({
      output$calibration <- renderDT({
        DF <- CalibrationData
        datatable(DF,
                  options = list(
                    scrollX = TRUE
                  ))
      })
    })
  })
  
  observe({recursiveExpInfo()})
  
  recursiveExpInfo <- eventReactive(input$expInfo,{
    updateTabsetPanel(session, "tabs", selected = "tab4")
  })
  
  observe({recursiveUploadIntens()})
  recursiveUploadIntens <- eventReactive(input$intensFile,{
    isolate({
      req(input$intensFile)
      DF <- tryCatch({
        read.csv(input$intensFile$datapath, header = TRUE,
                 check.names = TRUE)
      }, error = function(e) {
        showNotification("Invalid intensity file.", duration = 10, type = "error")
        return()
      })
      if (is.null(DF)) return()
      IntensData <<- DF
      output$intens <- renderDT({
        datatable(DF,
                  options = list(scrollX = TRUE)
                  )
      })
    })
  })
  
  observe({recursiveUploadExpFile()})
  recursiveUploadExpFile <- eventReactive(input$expFile,{
    isolate({
      req(input$expFile)
      DF <- tryCatch({
        read.csv(input$expFile$datapath, header = TRUE,
                 check.names = TRUE)
      }, error = function(e) {
        showNotification("Invalid experiment info file.", duration = 10, type = "error")
        return()
      })
      if (is.null(DF)) return()
      ExpInfo <<- DF
      MergedData <<- DF
      suppressWarnings(rm(CalibrationData, pos = 1))
      output$calibration <- renderDT({})
      
      output$experiment <- renderDT({
        datatable(DF,
                  options = list(
                    scrollX = TRUE
                  ))
      })
    })
  })
  
  observe({recursiveUploadPrepFile()})
  recursiveUploadPrepFile <- eventReactive(input$prepFile,{
    isolate({
      req(input$prepFile)
      DF <- tryCatch({
        read.csv(input$prepFile$datapath, header = TRUE,
                 check.names = TRUE)
      }, error = function(e) {
        showNotification("Invalid calibration data.", duration = 10, type = "error")
        return()
      })
      if (is.null(DF)) return()
      CalibrationData <<- DF
      MergedData <<- DF
      output$calibration <- renderDT({
        datatable(DF,
                  options = list(
                    scrollX = TRUE
                  ))
      })
      updateSelectInput(session = session, "concVar", choices = names(DF))
    })
  })
  
  observe({recursiveMerge()})
  recursiveMerge <- eventReactive(input$merge,{
    isolate({
      if (is.null(ExpInfo)) {
        showNotification("Experiment info not found.", duration=3, type="error")
      } else if (is.null(IntensData)) {
        showNotification("Intensity data not found.", duration=3, type="error")
      } else if (inherits(try(merge(ExpInfo, IntensData,
                                    by.x = input$mergeExp,
                                    by.y = input$mergeIntens, all = TRUE), silent = TRUE), "try-error")) {
        showNotification("Error in the column IDs.", duration=5, type="error")
      } else {
        DF <- merge(ExpInfo, IntensData,
                    by.x = input$mergeExp,
                    by.y = input$mergeIntens, all = TRUE)
        
        MergedData <<- DF
        CalibrationData <<- DF
        
        output$experiment <- renderDT({
          datatable(DF,
                    options = list(
                      scrollX = TRUE
                    ))
        })
      }
    })
  })
  
  observe({recursivePrepare()})
  recursivePrepare <- eventReactive(input$prepare,{
    DF <- MergedData
    CalibrationData <<- DF
    
    output$calibration <- renderDT({
      datatable(DF,
                options = list(
                  scrollX = TRUE
                ))
    })
    updateSelectInput(session, "concVar", choices = names(DF))
    updateTabsetPanel(session, "tabs", selected = "tab5")
  })
  
  observeEvent(input$combReps,{
    if (is.null(MergedData)) {
      showNotification("Calibration data not found.", duration = 10, type="error")
      return()
    }
    if (!input$combRepsColSI %in% colnames(MergedData)) {
      showNotification("Invalid column name in sample information.", duration = 10, type="error")
      return()
    }
    if (!input$combRepsColCL %in% colnames(MergedData)) {
      showNotification("Invalid column name in color information.", duration = 10, type="error")
      return()
    }
    Cols <- c(grep("Mean", colnames(MergedData)),
              grep("Median", colnames(MergedData)))
    RES <- NULL
    if(input$colorsBands > 1){
      DF <- MergedData[,c(input$combRepsColSI, input$combRepsColCL)]
      DFuni <- DF[!duplicated(DF),]
      for (i in 1:nrow(DFuni)) {
        sel <- DF[,1] == DFuni[i,1] & DF[,2] == DFuni[i,2]
        tmp <- MergedData[sel, ]
        tmp2 <- tmp[1, ]
        if (input$radioReps == 1) #mean
          tmp2[, Cols] <- colMeans(tmp[, Cols], na.rm = TRUE)
        if (input$radioReps == 2) #median
          tmp2[, Cols] <- apply(tmp[, Cols], 2, median, na.rm = TRUE)
        RES <- rbind(RES, tmp2)
      }
    }else{
      DF <- MergedData[,input$combRepsColSI]
      for (spl in unique(MergedData[, input$combRepsColSI])) {
        tmp <- MergedData[DF == spl, ]
        tmp2 <- tmp[1, ]
        if (input$radioReps == 1) #mean
          tmp2[, Cols] <- colMeans(tmp[, Cols], na.rm = TRUE)
        if (input$radioReps == 2) #median
          tmp2[, Cols] <- apply(tmp[, Cols], 2, median, na.rm = TRUE)
        RES <- rbind(RES, tmp2)
      }
    }
    rownames(RES) <- 1:nrow(RES)
    RES <- RES[order(RES[,input$combRepsColSI]),]
    CalibrationData <<- RES
    
    output$calibration <- renderDT({
      datatable(RES,
                options = list(
                  scrollX = TRUE
                ))
    })
  })
  
  observeEvent(input$reshapeWide,{
    if (is.null(MergedData)) {
      showNotification("Calibration data not found.", duration = 10, type="error")
      return()
    }
    if (!input$reshapeCol %in% colnames(MergedData) | !input$combRepsColSI2 %in% colnames(MergedData)) {
      showNotification("Unable to reshape data. Please check the specified column names and try again.", duration = 10, type="error")
      return()
    }
    rm.file <- (colnames(CalibrationData) != colnames(MergedData)[1] &
                  colnames(CalibrationData) != input$reshapeCol)
    DF.split <- split(CalibrationData[,rm.file], CalibrationData[,input$reshapeCol])
    
    N <- length(unique(CalibrationData[,input$reshapeCol]))
    if(N > 1){
      tryCatch({
        DF <- DF.split[[1]]
        Cols <- c(grep("Mean", colnames(DF)),
                  grep("Median", colnames(DF)))
        Cols <- c(Cols, which(colnames(DF) == input$combRepsColSI2))
        for(i in 2:N){
          DF <- merge(DF, DF.split[[i]][,Cols], by = input$combRepsColSI2,
                      suffixes = paste0(".", names(DF.split)[c(i-1,i)]))
        }
        CalibrationData <<- DF
      },
      error = function(e) {
        showNotification("Unable to reshape data. Please check the specified column names and try again.", duration = 10, type="error")
        return()
      })
    } else {
      DF <- CalibrationData
    }
    
    output$calibration <- renderDT({
      datatable(DF,
                options = list(
                  scrollX = TRUE
                ))
    })
  })
  
  MODELNUM <- 1
  
  observeEvent(input$runCali, {
    # flush the output and plots
    output$LOB <- renderText({})
    output$LOD <- renderText({})
    output$LOQ <- renderText({})
    output$plot5 <- renderPlot({})
    # Disable buttons
    shinyjs::disable("openReport")

    PATH.OUT <- input$folder
    if (!file.exists(PATH.OUT)) dir.create(PATH.OUT)
    
    concVar <- input$concVar
    respVar <- paste0("(",input$respVar,")")
    
    SUBSET <- input$subset
    if (SUBSET == "") {
      try_data <- CalibrationData 
    } else {
      try_data <- tryCatch({
        subset(CalibrationData, eval(parse(text = SUBSET)))
      }, error = function(e) {
        output$modelSummary <- renderPrint({
          print("Subsetting failed.")
          print("Please verify that the expression is valid and uses correct column names.")
        })
        updateTabsetPanel(session, "tabs", selected = "tab6")
        return()
      })
    }
    if (is.null(try_data)) return()
    if (nrow(try_data) < 2) {
      output$modelSummary <- renderPrint({
        print("Not enough data to fit the model (at least 2 rows required).")
      })
      updateTabsetPanel(session, "tabs", selected = "tab6")
      return()
    }
    
    # Check if log transform is possible and adapt the FORMULA
    if(input$useLog){
      if(any(try_data[[concVar]] <= 0)) {
        output$modelSummary <- renderPrint({
          print("Log transformation is not possible.")
          print("The concentration column contains zero or negative values.")
          print("Please check calibration data and try again.")})
        updateTabsetPanel(session, "tabs", selected = "tab6")
        return()
      }
      if(input$chosenModel == 3) {
        k <- ceiling(length(unique(CalibrationData[,concVar]))/2)
        FORMULA <- paste0(respVar, " ~ s(log10(", concVar, "), k = ", k, ")")  
      } else {
        FORMULA <- paste0(respVar, " ~ log10(", concVar, ")")  
      }
    } else {
      if(input$chosenModel == 3) {
        k <- ceiling(length(unique(CalibrationData[,concVar]))/2)
        FORMULA <- paste0(respVar, " ~ s(", concVar, ", k = ", k, ")")  
      }else{
        FORMULA <- paste0(respVar, " ~ ", concVar)
      }
    }
    
    # Check if model can be computed before creating the report.
    if(input$chosenModel == 1 && !inherits(try(lm(as.formula(FORMULA), data=try_data), silent = TRUE), "try-error")){
      modelName <- "lm"
      src <- system.file("markdown", "CalibrationAnalysis(lm).Rmd", package = "LFApp")
    } else if(input$chosenModel == 2 && !inherits(try(loess(as.formula(FORMULA), data = try_data), silent = TRUE), "try-error")){
      modelName <- "loess"
      src <- system.file("markdown", "CalibrationAnalysis(loess).Rmd", package = "LFApp")
    } else if(input$chosenModel == 3 && !inherits(try(gam(as.formula(FORMULA), data = try_data), silent = TRUE), "try-error")){
      modelName <- "gam"
      src <- system.file("markdown", "CalibrationAnalysis(gam).Rmd", package = "LFApp")
    } else {
      output$modelSummary <- renderPrint({print("Calibration can not be performed. Please check the formula.");
        print(paste0("Formula: ",FORMULA))})
      showNotification("Error in the formula!", duration = 5, type="error")
      updateTabsetPanel(session, "tabs", selected = "tab6")
      return()
    }
      
    info <- showNotification(paste("Fitting the model..."), duration = 0, type="message")

    FILENAME <<- paste0(format(Sys.time(), "%Y%m%d_%H%M%S_"), input$analysisName)

    header <- c('---',
                'title: "Calibration Analysis"',
                'date: "`r format(Sys.time(), \'%d %B %Y\')`"',
                'output:',
                '  rmarkdown::html_document:',
                '    self_contained: true',
                '    theme: united',
                '    highlight: tango',
                '    toc: true',
                '    number_sections: true',
                'params:',
                paste0('  filename: ', file.path(PATH.OUT, FILENAME)),
                paste0('  model: ', modelName),
                paste0('  concVar: ', concVar),
                paste0('  formula: ', FORMULA),
                if (modelName == "gam") paste0('  k: ', k),
                if (SUBSET != "") paste0('  subset: ', SUBSET),
                '---')

    save(CalibrationData, FORMULA, SUBSET, PATH.OUT,
         file = file.path(PATH.OUT, paste0(FILENAME, "_Data.RData")))
    template <- readLines(src)
    write(header, file=file.path(PATH.OUT, paste0(FILENAME, "_Analysis.Rmd")), append=FALSE)
    write(template, file=file.path(PATH.OUT, paste0(FILENAME, "_Analysis.Rmd")), append=TRUE)
    reportGenerated <- tryCatch({
      rmarkdown::render(input = file.path(PATH.OUT, paste0(FILENAME, "_Analysis.Rmd")),
                        output_file = file.path(PATH.OUT, paste0(FILENAME, "_Analysis.html")))
      TRUE
    }, error = function(e) {
      print(e)
      removeNotification(info)
      output$modelSummary <- renderPrint({
        print("Something went wrong while generating the report.")
        print("Review your calibration data for errors and try again.")
        print("Advanced users can check ReportAnalysis.Rmd to investigate further")
      })
      updateTabsetPanel(session, "tabs", selected = "tab6")
      FALSE
    })
    if (reportGenerated != TRUE) return()
    
    output$modelSummary <- renderPrint({ fit })
    
    output$plot5 <- renderPlot({
      modelPlot
    })
    output$LOB <- renderText({
      paste0("Limit of Blank (LOB): ", signif(LOB, 3))
    })
    output$LOD <- renderText({
      paste0("Limit of Detection (LOD): ", signif(LOD, 3))
    })
    output$LOQ <- renderText({
      paste0("Limit of Quantification (LOQ): ", signif(LOQ, 3))
    })
      
    # Adding the analysis name and model formula to the table
    modelName <- rep(modelName, nrow(CalibrationData))
    modelFormula <- rep(FORMULA, nrow(CalibrationData))
    modelDF <- cbind(modelName, modelFormula, predFunc(CalibrationData))
    colnames(modelDF) <- c(paste0(input$analysisName, ".model"), 
                           paste0(input$analysisName, ".formula"), 
                           paste0(input$analysisName, ".", input$concVar, ".fit"))
    if(SUBSET != ""){
      subsetIndex <- function (x, subset){
        e <- substitute(subset)
        r <- eval(e, x, parent.frame())
        r & !is.na(r)
      }
      Index <- eval(call("subsetIndex", x = CalibrationData, 
                         subset = parse(text = SUBSET)))
      modelDF[!Index,] <- NA
    }
    DF <- cbind(CalibrationData, modelDF)
    CalibrationData <<- DF
    output$calibration <- renderDT({
      datatable(DF,
                options = list(
                  scrollX = TRUE,
                  autowidth = TRUE
                ))
    })
        
    MODELNUM <<- MODELNUM + 1
      
    updateTextInput(session=session, inputId="analysisName", value=paste0("Model", MODELNUM))
    
    removeNotification(info)
    showNotification(paste("Calibration completed successfully."), duration = 5, type="message")
    shinyjs::enable("openReport")

    updateTabsetPanel(session, "tabs", selected = "tab6")
  })
  
  observe(resetFolder())
  
  resetFolder <- eventReactive(input$folder,{
    isolate({
      if(substring(input$folder,1,nchar(file.path(fs::path_home()))) != file.path(fs::path_home()))
        updateTextInput(session=session, inputId = "folder", value = file.path(fs::path_home())) 
    })
  })
  
  observe({recursiveOpenReport()})
  
  recursiveOpenReport <- eventReactive(input$openReport,{
    isolate({
      browseURL(file.path(input$folder, paste0(FILENAME, "_Analysis.html")),
                browser = getOption("browser"))
    })
  })
  
  #creates the textbox below plot2 about the plot_brush details and etc
  output$thresh <- renderText({
    if(!is.null(shinyImageFile$Threshold))
      paste0("Threshold(s): ", paste0(signif(shinyImageFile$Threshold, 4), collapse = ", "))
  })
  output$meanIntens <- renderText({
    if(!is.null(shinyImageFile$Threshold))
      paste0("Mean intensities: ", paste0(signif(shinyImageFile$Mean_Intensities, 4), collapse = ", "))
  })
  output$medianIntens <- renderText({
    if(!is.null(shinyImageFile$Threshold))
      paste0("Median intensities: ", paste0(signif(shinyImageFile$Median_Intensities, 4), collapse = ", "))
  })
  output$intens <- renderDT({
    DF <- IntensData
    datatable(DF)
  })
  output$folder <- renderPrint({
    paste0("Folder for Results: ", parseDirPath(c(wd=fs::path_home()), input$folder))
  })
  
  #allows user to download data
  output$downloadData <- downloadHandler(
    filename = "IntensityData.csv",
    content = function(file) {
      write.csv(IntensData, file, row.names = FALSE)
    }
  )
  output$downloadData2 <- downloadHandler(
    filename = "MergedData.csv",
    content = function(file) {
      write.csv(MergedData, file, row.names = FALSE)
    }
  )
  output$downloadData3 <- downloadHandler(
    filename = "CalibrationData.csv",
    content = function(file) {
      write.csv(CalibrationData, file, row.names = FALSE)
    }
  )
  
  #When user clicks the return to command line button
  #stops the shiny app
  # prevents user from quitting shiny using ^C on commandline
  observe({recursiveStop()})
  
  recursiveStop <- eventReactive(input$stop,{
    isolate({
      suppressWarnings(rm(IntensData, pos = 1))
      suppressWarnings(rm(ExpInfo, pos = 1))
      suppressWarnings(rm(MergedData, pos = 1))
      suppressWarnings(rm(CalibrationData, pos = 1))
      stopApp()
    })
  })
  
  # Checking if workspace file exist
  observe({
    if (file.exists(file.path(fs::path_home(), "Documents/LFApp/cal_autosave.RData"))) {
      showModal(modalDialog(
        title = "Old workspace backup found",
        "Do you want to restore previous workspace?",
        h6("If you do not restore the old workspace, it will be overwritten!"),
        footer = tagList(
          actionButton("no_restore", "No"),
          actionButton("restore_work", "Yes")
        )
      ))
    } else {
      startAutosave(TRUE)
    }
  })
  
  observeEvent(input$no_restore, {
    startAutosave(TRUE)
    removeModal()
  })
  
  observeEvent(input$restore_work, {
    load(file=file.path(fs::path_home(), "Documents/LFApp/cal_autosave.RData"))
    
    # Loading the variables properly. Without this the variables are loaded in the observe scope only
    shinyImageFile <<- shinyImageFile
    IntensData <<- IntensData
    ExpInfo <<- ExpInfo
    MergedData <<- MergedData
    fit <<- fit
    modelPlot <<- modelPlot
    LOB <<- LOB
    LOD <<- LOD
    LOQ <<- LOQ
    
    # Loading the image on the first plot
    if (!is.null(shinyImageFile$shiny_img_final))
      output$plot1 <- renderPlot({EBImage::display(shinyImageFile$shiny_img_final, method = "raster")})
    
    # Loading datatables
    output$intens <- renderDT(datatable(IntensData))
    output$experiment <- renderDT(datatable(ExpInfo))
    output$calibration <- renderDT(datatable(MergedData))
    
    # Loading model in results tab
    if (!is.null(fit) && !is.null(LOB) && !is.null(LOD) && !is.null(LOQ)) {
      output$modelSummary <- renderPrint({ fit })
      output$plot5 <- renderPlot({ modelPlot })
      output$LOB <- renderText({ paste0("Limit of Blank (LOB): ", signif(LOB, 3)) })
      output$LOD <- renderText({ paste0("Limit of Detection (LOD): ", signif(LOD, 3)) })
      output$LOQ <- renderText({ paste0("Limit of Quantification (LOQ): ", signif(LOQ, 3)) })
    }
    startAutosave(TRUE)
    removeModal()
  })
  
  # Autosaving every minute
  observe({
    if (startAutosave()) {
      invalidateLater(300000, session)
      if (!file.exists(file.path(fs::path_home(), "Documents/LFApp"))) dir.create(file.path(fs::path_home(), "Documents/LFApp"))
      save(shinyImageFile, IntensData, 
           ExpInfo, MergedData, fit, 
           modelPlot, LOB, LOD, LOQ,
           file=file.path(fs::path_home(), "Documents/LFApp/cal_autosave.RData"))
      showNotification("Workspace saved", duration=2, type="message")
    }
  })
  
  # A function to remove the autosave file if the app was closed properly
  # onStop(function() file.remove(file.path(fs::path_home(), "Documents/LFApp/cal_autosave.RData")))
}

triangle <- function(image, offset = 0.2, breaks = 256) {
  if(!is(image, "Image"))
    stop("'image' must be of class 'Image'!")
  breaks <- as.integer(breaks)
  stopifnot(offset >= 0)
  stopifnot(breaks > 0)
  
  ## compute histogram and extract counts and breaks
  rg <- range(image)
  bins <- breaks
  breaks <- seq(rg[1], rg[2], length = breaks + 1L)
  image.hist <- hist.default(imageData(image), breaks = breaks, plot = FALSE)
  hist.counts <- image.hist$counts
  hist.breaks <- image.hist$breaks
  
  ## centers of bins
  delta <- hist.breaks[2]-hist.breaks[1]
  hist.bins <- hist.breaks[-(bins+1)] + delta/2
  
  ## location of peaks and peak value
  ind.peaks <- which(hist.counts == max(hist.counts))
  ind.first.peak <- ind.peaks[1]
  ind.last.peak <- ind.peaks[length(ind.peaks)]
  peak.height <- hist.counts[ind.first.peak]
  
  ## fist and last bin with positive count
  pos.counts <- which(hist.counts > 0)
  ind.low <- pos.counts[1]
  ind.high <- pos.counts[length(pos.counts)]
  
  if((ind.first.peak - ind.low) < (ind.high - ind.last.peak)){
    ## right tail is longer
    sel <- ind.last.peak:ind.high
    norm.counts <- hist.counts[sel]/peak.height
    norm.bins <- (1:length(sel))/length(sel)
    distances <- (1-norm.counts)*(1-norm.bins)/sqrt((1-norm.counts)^2 + (1-norm.bins)^2)
  }else{
    ## left tail is longer
    sel <- ind.low:ind.first.peak
    norm.counts <- hist.counts[sel]/peak.height
    norm.bins <- (1:length(sel))/length(sel)
    distances <- (1-norm.counts)*norm.bins/sqrt((1-norm.counts)^2 + norm.bins^2)
  }
  ind.max <- which.max(distances)
  
  hist.bins[sel[ind.max]] + offset
}

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE, warning = FALSE, message = FALSE, error = FALSE)
```

# Import of Data

We import the data and select the variables needed for the analysis.
```{r}
load(paste0(params$filename, "_Data.RData"))
tmp <- strsplit(params$formula, " ~ ")[[1]]

y.var <- names(unlist(sapply(colnames(CalibrationData), 
                             grep, x = tmp[1])))
x.var <- names(unlist(sapply(colnames(CalibrationData), 
                              grep, x = tmp[2])))

if (!is.null(params$subset)) {
  calData <- subset(CalibrationData, eval(parse(text = params$subset)))
} else {
  calData <- CalibrationData
}
calData <- calData[,c(y.var, x.var)]
calData
```


# Model

We will apply the following model.
```{r}
params$formula
```


# Analysis

We now fit (simple) linear model.
```{r}
fit <- lm(as.formula(params$formula), data = calData)
summary(fit)
```

We determine the inverse of the fitted model.
```{r}
ab <- coef(fit)
predFunc <- function(newdata){}
body(predFunc) <- substitute({ with(newdata, (eval(y)-a)/b) }, 
                             list(y = parse(text = tmp[1]), 
                                a = ab[[1]], 
                                b = ab[[2]]))
```

We plot the given concentrations against the fitted values.
```{r, warning=FALSE}
library(ggplot2)
modelPlot <- ggplot(calData, aes(x = .data[[params$concVar]],
                                 y = !!rlang::parse_expr(tmp[1]))) +
            geom_point() + geom_smooth(method = "lm") +
            annotate("text",  x=-Inf, y = Inf,
            label = paste0("R² = ", signif(summary(fit)$r.squared, 3),
                           ", adj. R² = ", signif(summary(fit)$adj.r.squared, 3)),
            vjust=1, hjust=0, size=5)
modelPlot
```

# Computation of LOB, LOD and LOQ

We compute limit of blank (LOB), limit of detection (LOD) and limit of 
quantification (LOQ) by inverting the regression fit. We get the LOB by inverting 
the upper bound of the one-sided 95\% confidence interval at concentration $0$. 
In case of LOD, the upper-bound of the 99.95\% confidence interval at 
concentration $0$ is inverted. LOQ is determined as $3\times\textrm{LOD}$.
```{r}
if(ab[2] > 0){
  LOB <- (confint(fit, parm = 1, level = 0.90)[2]-ab[1])/ab[2]
  LOD <- (confint(fit, parm = 1, level = 0.999)[2]-ab[1])/ab[2]
}else{
  LOB <- (confint(fit, parm = 1, level = 0.90)[1]-ab[1])/ab[2]
  LOD <- (confint(fit, parm = 1, level = 0.999)[1]-ab[1])/ab[2]
}
names(LOB) <- "LOB"
names(LOD) <- "LOD"
LOQ <- 3*LOD
names(LOQ) <- "LOQ"
LOB
LOD
LOQ
```

# Save Results

We save the results.
```{r}
save(fit, LOB, LOD, LOQ, file = paste0(params$filename, "_Results.RData"))
```

# Save Model

We save the inverse of the fitted model to be able to apply it for predicting
concentrations.
```{r}
saveRDS(object = predFunc, file = paste0(params$filename, "_Model.rds"))
```

# Software

```{r}
sessionInfo()
```

 
```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE, warning = FALSE, message = FALSE, error = FALSE)
```

# Import of Data

We import the data and select the variables needed for the analysis.
```{r}
load(paste0(params$filename, "_Data.RData"))
tmp <- strsplit(params$formula, " ~ ")[[1]]

y.var <- names(unlist(sapply(colnames(CalibrationData), 
                             grep, x = tmp[1])))
x.var <- names(unlist(sapply(colnames(CalibrationData), 
                              grep, x = tmp[2])))
if (!is.null(params$subset)) {
  calData <- subset(CalibrationData, eval(parse(text = params$subset)))
} else {
  calData <- CalibrationData
}
calData <- calData[,c(y.var, x.var)]
calData
```


# Model

We will apply the following model.
```{r}
params$formula
```


# Analysis

We now fit local polynomial model.
```{r}
fit <- loess(as.formula(params$formula), data = calData)
summary(fit)
```

We compute the predicted concentration values and determine the inverse of the 
fitted model by spline interpolation.
```{r}
N <- 100 # number of points for interpolation
conc <- seq(from = min(calData[,params$concVar]), to = max(calData[,params$concVar]), len = N)
newDF <- data.frame(conc = conc)
names(newDF) <- params$concVar
resp <- as.vector(predict(fit, newdata = newDF))
predFunc <- function(newdata){}
body(predFunc) <- substitute({ Y <- with(newdata, eval(y))
                         fun <- splinefun(x = resp, y = conc, method = "monoH.FC")
                         fun(Y) }, list(y = parse(text = tmp[1]),
                                        resp = resp,
                                        conc = conc))
  
y0 <- predict(fit, newdata = newDF[1,,drop = FALSE], se = TRUE)
fun <- splinefun(x = resp, y = conc, method = "monoH.FC")
```

We plot the given concentrations against the fitted values.
```{r}
library(ggplot2)
modelPlot <- ggplot(calData, aes(x = .data[[params$concVar]],
                                 y = !!rlang::parse_expr(tmp[1]))) +
                geom_point() + geom_smooth(method = "loess")
modelPlot
```

# Computation of LOB, LOD and LOQ

We compute limit of blank (LOB), limit of detection (LOD) and limit of 
quantification (LOQ) by inverting the regression fit. We get the LOB by inverting 
the upper bound of the one-sided 95\% confidence interval at concentration $0$. 
In case of LOD, the upper-bound of the 99.95\% confidence interval at 
concentration $0$ is inverted. LOQ is determined as $3\times\textrm{LOD}$.
```{r}
if(resp[1] < resp[length(resp)]){
  LOB <- fun(resp[1] + qt(0.95, df = y0$df)*y0$se.fit)
  LOD <- fun(resp[1] + qt(0.9995, df = y0$df)*y0$se.fit)
}else{
  LOB <- fun(resp[1] - qt(0.95, df = y0$df)*y0$se.fit)
  LOD <- fun(resp[1] - qt(0.9995, df = y0$df)*y0$se.fit)
}
names(LOB) <- "LOB"
names(LOD) <- "LOD"
LOQ <- 3*LOD
names(LOQ) <- "LOQ"
LOB
LOD
LOQ
```

# Save Results

We save the results.
```{r}
save(fit, LOB, LOD, LOQ, file = paste0(params$filename, "_Results.RData"))
```

# Save Model

We save the inverse of the fitted model to be able to apply it for predicting
concentrations.
```{r}
saveRDS(object = predFunc, file = paste0(params$filename, "_Model.rds"))
```

# Software

```{r}
sessionInfo()
```

---
title: "LFA App User's Guide"
author: "Filip Paskali, Weronika Schary, Matthias Kohl"
date: "`r Sys.Date()`"
output: 
  rmarkdown::html_vignette:
    toc: true
bibliography: lit.bib
vignette: >
  %\VignetteIndexEntry{LFA App User's Guide}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{utf8}
---


<br><br>

# Introduction

The LFA shiny apps [@Shiny2020] consist of four modular Shiny applications: 

(1) **LFA App core** for image acquisition, editing, region of interest definition 
via gridding, background correction with multiple available methods, as well as 
intensity data extraction of the pre-defined bands of the analysed LFAs. More
precisely, it consists of Tab 1, Tab 2 and parts of Tab 3 described in detail 
below.

(2) **LFA App calibration** extends the LFA App core by methods for merging the
intensity data with information from experiments, computation of calibration 
models and the generation of a report about the calibration analysis. The 
functionality corresponds to the Tabs 1-6 described below.

(3) **LFA App quantification** enables quantification of the extracted intensity 
values via loading existing calibration models. It extends the LFA App core
by Tab 7 described below.

(4) **LFA App analysis** includes the full functionality mentioned above and 
enables full analysis in one application. That is, it consists of Tab 1-7.


The graphical user interface of the apps is built in a modular way divided into 
several tabs, where each tab represents a specific step of the workflow. While 
the applications can be used in a sequential fashion, the specific steps can 
also be carried out individually. 

<br><br>

# Testing our apps without installing

Before installing our package, one can also test our apps on https://www.shinyapps.io. 
The desktop version of our full purpose analysis app is at

https://lfapp.shinyapps.io/LFAnalysis/

The respective mobile version is at

https://lfapp.shinyapps.io/mobile_app/

<br><br>

# Installation

The package requires Bioconductor package EBImage, which should be installed
first via.

```{r, eval = FALSE}
## Install package BiocManager
if(!requireNamespace("BiocManager", quietly = TRUE)) 
  install.packages("BiocManager")
## Use BiocManager to install EBImage
BiocManager::install("EBImage", update = FALSE)
```

Our package depends on the most recent version of package shinyMobile, which 
must be installed from github (https://github.com/RinteRface/shinyMobile) by

```{r, eval = FALSE}
## Install package remotes
if(!requireNamespace("remotes", quietly = TRUE)) 
  install.packages("remotes")
## Install package shinyMobile
remotes::install_github("RinteRface/shinyMobile")
```

For generating our vignette and automatic reports, we need packages knitr and
rmarkdown, which will be installed next.

```{r, eval = FALSE}
## Install package knitr
if(!requireNamespace("knitr", quietly = TRUE)) 
  install.packages("knitr")
## Install package rmarkdown
if(!requireNamespace("rmarkdown", quietly = TRUE)) 
  install.packages("rmarkdown")
```

Finally, one can install package LFApp, where all remaining dependencies will
be installed automatically.

```{r, eval = FALSE}
## Install package LFApp
remotes::install_github("fpaskali/LFApp", build_vignette=TRUE)
```

<br><br>

# Start Apps

As already described above, package LFApp includes four different shiny apps 
where there is a desktop and a mobile version for each app. They can be started 
with one of the following commands: 

```{r, eval = FALSE}
## desktop versions
## LFA App core
LFApp::run_core()

## LFA App quantification
LFApp::run_quan()

## LFA App calibration
LFApp::run_cal()

## LFA App full analysis
LFApp::run_analysis()

## mobile versions
## LFA App core
LFApp::run_mobile_core()

## LFA App quantification
LFApp::run_mobile_quan()

## LFA App calibration
LFApp::run_mobile_cal()

## LFA App full analysis
LFApp::run_mobile_analysis()
```

<br><br>

# Tab 1: Cropping and Segmentation

## Upload Image or Choose Sample Image

The first step consists of loading an image of one or several lateral flow strips. 
For trying out the app one can also load a sample image provided with the package.
For the purpose of demonstration we will use the sample image provided with the 
package. The image can be rotated or flipped horizontally (button FH) and 
vertically (button FV) via the rotation panel on the left hand side. It can also 
be cropped to a specific size via double click on the interactive plot.


## Set number of strips and number of lines per strip

In the next step, one needs to select the number of strips shown in the image and 
the number of lines (bands) per strip.

The maximum number of strips per image is 10, but in principle could be extended 
to a higher number. The only requirements is that the strips are regularly spaced
with identical or at least similar spaces between them and between the lines. 
The minimum number of lines per strip is two, the maximum number of lines per 
strip is six, since we are not aware of any lateral flow assay having more than 
six lines. But again, this could be easily extended to a higher number.


## Apply Segmentation

After loading the image and selecting the number of strips as well as lines per strip, 
one needs to click on the image and drag to generate a rectangular selection region 
on the image. It is best, if each line is roughly in the middle of a respective segment. 
In addition, there is a segment between two consecutive bands. This segment should 
not include any part of a line, since it can be used for background correction; 
see below. An example of a well selected cropping region is shown in the following 
screenshot. 

<p>![](LFApp.png){width=100%}</p>

By clicking "APPLY SEGMENTATION" the original image is cropped to size of the 
selected region and segmented. The segments between the bands are used in our 
quantile background correction method; see below. If you want to change the 
cropped region you can select "RESET". 

The cropping was adapted from the shiny app provided by package ShinyImage 
[@shinyImage].

<br><br>

# Tab 2: Background

## Select Strip

If there is more than one strip on the analysed image, one first needs to select, 
which strip shall be analysed.


## Optional: in case of color images

If the image is a color image, it will be transformed to grayscale. By default,
we apply the luminance approach, which converts color to grayscale preserving
luminance; that is, the grayscale values are obtained by 

$$
  0.2126 * R + 0.7152 * G + 0.0722 * B
$$
where R, G, B stands for the red, green and blue channel of the color image. 
By selecting mode "gray", the arithmetic mean of the RGB values is used. 
Furthermore, the selection of mode "red", "green" or "blue" allows a color 
channel wise analysis of color images.


## Select threshold method and apply

There are four threshold methods, where the default is Otsu's method [@Otsu.1979].
Otsu's method returns a single intensity threshold that separates pixels into 
foreground and background. It is equivalent to k-means clustering of the intensity 
histogram [@Liu.2009]. Otsu and Li [@Li.1993, @Li.1998] are non-parametric, 
fully automatic algorithms that find the optimal threshold for the image. 
Additionally, two semi-automatic algorithms, namely Quantile and Triangle 
[@Zack.1977] were included, to cover different images and cases where automatic 
threshold results are not ideal. 
The quantile method is a simple method that computes the specified quantile of
all pixel intensities of the segments between the lines (per strip). In most of 
the images we have analysed so far, Otsu's method performed very well and better 
than our quantile method. 
However, in cases where the lines are unclear and very blurred our quantile method 
may outperform Otsu's method. In case of the triangle method an additional 
offeset (default = 0.2) is added to the computed threshold.


By clicking "APPLY TRESHOLD" the selected threshold method is applied to the 
segmented images of the selected strip. The upper plots show the pixels that have 
intensities above the background. The lower plots show the images after background 
subtraction; see the screenshot below.

<p>![](BackgroundCorrection.png){width=100%}</p>

The thresholds (top of the page) as well as calculated mean and median intensities 
of the lines (bottom of the page) in order from top of the strip to the bottom 
of the strip are shown as well.


## Add to intensity data

Clicking "ADD TO INTENSITY DATA" adds the mean and median of the background 
subtracted intensities of the pixels with intensities above the threshold to the 
data. Now, one can proceed with the second strip of the image, use a different 
color conversion mode or go back to Tab 1 and load the next image. When all strips 
and images are processed one can proceed with "SWITCH TO INTENSITY DATA". 


## Switch To Intensity Data

Clicking "SWITCH TO INTENSITY DATA" changes the Tab to Tab 3.

<br><br>

# Tab 3: Intensity Data

Here, the latest version of the generated intensity data is shown in the Tab.


## Download data

By clicking "DOWNLOAD DATA" the data can be saved as a standard csv file. 


## Restart with new data

We also provide a "DELETE DATA" button for restarting with new images or 
uploading an already existing dataset. 


## Upload existing data

Instead of generating new data, one can also upload already existing intensity 
data, that can also be generated with a different software. This step can also 
be seen as a second entry point of the app. The screenshot below shows an example 
of data that was generated with the app saved and then loaded again.

<p>![](IntensityData.png){width=100%}</p>


## Switch To Experiment Info

Clicking "SWITCH TO EXPERIMENT INFO" changes the Tab to Tab 4, where information 
about the experiment can be loaded.

<br><br>

# Tab 4: Experiment Info

## Upload Experiment Info or upload existing merged data and go to calibration

One first can either upload information about the experiment in form of a .csv file 
or upload already merged data (intensity data merged with experiment information). 
An example of a table with information about the experiment is shown in the 
screenshot below.

<p>![](ExperimentData.png){width=100%}</p>

This is a third optional starting point of the app. Here, one can also directly 
start with already merged data and proceed with the calibration analysis.


## Select ID columns and merge datasets

For merging the intensity data with the information about the experiment, one 
has to specify the names of the columns on which the merge should take place. 
The default is "File", as a "File" column is generated when the intensity data 
is computed with the app. By clicking "MERGE WITH INTENSITY DATA" the two 
datasets will be merged. An example of a table with merged data is shown in the 
screenshot below.

<p>![](ExperimentMergedData.png){width=100%}</p>


## Download data

The merged data can be downloaded as a .csv file by clicking "DOWNLOAD DATA".


## Restart with new data

We again provide a "DELETE DATA" button for restarting with new images, uploading 
new intensity data on Tab 3 or uploading an already existing merged dataset.


## Prepare Calibration

Clicking "PREPARE CALIBRATION" changes the Tab to Tab 5, where the data can be 
further preprocessed and the calibration analysis can be started.

<br><br>

# Tab 5: Calibration

## Specify a folder for the analysis results

In the first step, a folder for the analysis results should be specified. If no 
folder is provided, a new folder "LFApp" will be generated in the home directory 
of the user. 


## Upload existing preprocessed data

Instead of generating the preprocessed data with our app, one can also upload 
preprocessed data and perform the calibration analysis on them. An example of 
uploaded preprocessed data that were generated with our app and saved as a .csv 
file is shown in the following screenshot.

<p>![](Opt1Calibration.png){width=100%}</p>


## Optional: average technical replicates

If the merged data includes technical replicates, these replicates can be averaged 
either by the arithmetic mean or the median. For this purpose the name of the column, 
which enables the identification of the replicates is required. By default we assume 
that this column is called "Sample". In case there is more than one analyte/color 
per line/band, one should set "Number of analytes/colors per line" to the respective 
number. If this number is larger than 1, an additional field will be shown, where 
one should enter the name of the column including the analyte/color information.


## Optional: reshape data from long to wide

If there is more than one analyte/color per line/band, one might want to combine 
both information into one calibration model. For such cases it is useful to reshape 
the data from long (analyte/color data in the rows) to wide (analyte/color data 
in the columns).

<p>![](Opt2Calibration.png){width=100%}</p>


## Download data

By clicking "DOWNLOAD DATA", the data used for the calibration can be downloaded 
as .csv file. 


## Calibration by linear, local polynomial or generalized additive model

In the final step of this Tab, one has to specify the calibration model. 
The model can the chosen via a checkbox (linear (lm), local polynomial (loess), 
generalized additive model (GAM)). A column for concentration values has to be 
selected. The concentration values can be logarithmized. In addition, one has
to specify a response variable. 

<p>![](Calibration.png){width=100%}</p>

If only a subset of the data shall be used for the calibration analysis, an 
logical R expression can be specified in the field below 
"Optional: specify subset (logical R expression)". This expression will be applied 
to select the respective subset.


## Run calibration analysis

By pressing "RUN CALIBRATION ANALYSIS" the RData-file "CalibrationData.RData"
will be generated and saved in the specified analysis folder. Next, the R markdown
file "CalibrationAnalysis(model).Rmd", where model = lm, loess, gam, will be 
copied to the same folder and will be rendered into file 
"CalibrationAnalysis(model).html" by using R package "rmarkdown" [@rmarkdown]. 
The results of the calibration analysis will be saved in file 
"CalibrationResults.RData" and will be used to generated a brief overview
of the results that will be shown on Tab 6. All files will remain in the
folder selected for the analysis results. That is, one may use and modify these 
files for an extended analysis. In addition, the fitted values for 
the concentrations are added to data used for the calibration. That is, one
can also run several different calibration models, download the calibration 
data and use the data to compare the results of different calibration models.

<br><br>

# Tab 6: Results

This Tab shows a brief overview of the analysis results as shown in the following 
screenshot.

<p>![](Results.png){width=100%}</p>


## Open analysis report

By clicking "Open" the full report of the calibration analysis will be opened
in the standard browser. The first part of the report is visible in the following
screenshot.

<p>![](AnalysisReport.png){width=100%}</p>

<br><br>

# Tab 7: Quantification

The final tab of our application enables the quantification of intensity values 
with the help of a calibration model. You can upload an existing model from a 
previous calibration analysis and use it to predict the concentrations for the 
intensity values acquired beforehand by clicking "PREDICT". Via the "DOWNLOAD DATA" 
button the data table can be saved as a .csv file containing the intensity values 
as well as the predicted concentrations.

<p>![](Quantification.png){width=100%}</p>

<br><br>

# Required R packages

Our app requires the following R packages to work properly:

* stats: The R Stats Package [@R2020]
* mgcv: Mixed GAM Computation Vehicle with Automatic Smoothness Estimation [@mgcv]
* shiny: Web Application Framework for R [@Shiny2020]
* shinyjs: Easily Improve the User Experience of Your Shiny Apps in Seconds [@shinyjs.2020]
* shinythemes: Themes for Shiny [@shinythemes.2018]
* fs: Cross-Platform File System Operations Based on 'libuv' [@fs.2020]
* shinyFiles: A Server-Side File System Viewer for Shiny [@shinyfiles.2020]
* Bioconductor package EBImage: R package for image processing with applications to cellular phenotypes [@Pau.2010]
* DT: A Wrapper of the JavaScript Library 'DataTables' [@DT.2020]
* rmarkdown: Dynamic Documents for R [@rmarkdown]
* ggplot2: Elegant Graphics for Data Analysis [@Wickham.2016]

<br><br>

# References

---
title: "LFA App User's Guide"
author: "Filip Paskali, Weronika Schary, Matthias Kohl"
date: "`r Sys.Date()`"
output: 
  rmarkdown::html_vignette:
    toc: true
bibliography: lit.bib
vignette: >
  %\VignetteIndexEntry{LFA App User's Guide}
  %\VignetteEncoding{utf8}
  %\VignetteEngine{knitr::rmarkdown}
---

<br><br>

# Introduction

The LFA shiny apps [@Shiny2020] consist of four modular Shiny applications:

(1) **LFA App core** for image acquisition, editing, region of interest definition via gridding, background correction with multiple available methods, as well as intensity data extraction of the pre-defined bands of the analysed LFAs. More precisely, it consists of Tab 1, Tab 2 and parts of Tab 3 described in detail below.

(2) **LFA App calibration** extends the LFA App core by methods for merging the intensity data with information from experiments, computation of calibration models and the generation of a report about the calibration analysis. The functionality corresponds to the Tabs 1-6 described below.

(3) **LFA App quantification** enables quantification of the extracted intensity values via loading existing calibration models. It extends the LFA App core by Tab 7 described below.

(4) **LFA App analysis** includes the full functionality mentioned above and enables full analysis in one application. That is, it consists of Tab 1-7.

The graphical user interface of the apps is built in a modular way divided into several tabs, where each tab represents a specific step of the workflow. While the applications can be used in a sequential fashion, the specific steps can also be carried out individually.

<br><br>

# Testing our apps without installing

Before installing our package, one can also test our apps on <https://www.shinyapps.io>. The desktop version of our full purpose analysis app is at

<https://lfapp.shinyapps.io/LFAnalysis/>

The respective mobile version is at

<https://lfapp.shinyapps.io/mobile_app/>

<br><br>

# Installation

The package requires Bioconductor package EBImage, which should be installed first via.

```{r, eval = FALSE}
## Install package BiocManager
if(!requireNamespace("BiocManager", quietly = TRUE)) 
  install.packages("BiocManager")
## Use BiocManager to install EBImage
BiocManager::install("EBImage", update = FALSE)
```

For generating our vignette and automatic reports, we need packages knitr and rmarkdown, which will be installed next.

```{r, eval = FALSE}
## Install package knitr
if(!requireNamespace("knitr", quietly = TRUE)) 
  install.packages("knitr")
## Install package rmarkdown
if(!requireNamespace("rmarkdown", quietly = TRUE)) 
  install.packages("rmarkdown")
```

Finally, one can install package LFApp, where all remaining dependencies will be installed automatically.

```{r, eval = FALSE}
## Install package remotes
if(!requireNamespace("remotes", quietly = TRUE)) 
  install.packages("remotes")
## Install package LFApp
remotes::install_github("fpaskali/LFApp", build_vignette=TRUE)
```

<br><br>

# Start Apps

As already described above, package LFApp includes four different shiny apps where there is a desktop and a mobile version for each app. They can be started with one of the following commands:

```{r, eval = FALSE}
## desktop versions
## LFA App core
LFApp::run_core()

## LFA App quantification
LFApp::run_quan()

## LFA App calibration
LFApp::run_cal()

## LFA App full analysis
LFApp::run_analysis()

## mobile versions
## LFA App core
LFApp::run_mobile_core()

## LFA App quantification
LFApp::run_mobile_quan()

## LFA App calibration
LFApp::run_mobile_cal()

## LFA App full analysis
LFApp::run_mobile_analysis()
```

<br><br>

# Tab 1: Cropping and Segmentation

## Upload Image or Choose Sample Image

The first step consists of loading an image of one or several lateral flow strips. For trying out the app one can also load a sample image provided with the package. For the purpose of demonstration we will use the sample image provided with the package. The image can be rotated or flipped horizontally (button FH) and vertically (button FV) via the rotation panel on the left hand side. It can also be cropped to a specific size via double click on the interactive plot.

## Set number of strips and number of lines per strip

In the next step, one needs to select the number of strips shown in the image and the number of lines (bands) per strip.

The maximum number of strips per image is 10, but in principle could be extended to a higher number. The only requirements is that the strips are regularly spaced with identical or at least similar spaces between them and between the lines. The minimum number of lines per strip is two, the maximum number of lines per strip is six, since we are not aware of any lateral flow assay having more than six lines. But again, this could be easily extended to a higher number.

## Apply Segmentation

After loading the image and selecting the number of strips as well as lines per strip, one needs to click on the image and drag to generate a rectangular selection region on the image. It is best, if each line is roughly in the middle of a respective segment. In addition, there is a segment between two consecutive bands. This segment should not include any part of a line, since it can be used for background correction; see below. An example of a well selected cropping region is shown in the following screenshot.

<p>![](LFApp.png){width="100%"}</p>

By clicking "APPLY SEGMENTATION" the original image is cropped to size of the selected region and segmented. The segments between the bands are used in our quantile background correction method; see below. If you want to change the cropped region you can select "RESET".

The cropping was adapted from the shiny app provided by package ShinyImage [@shinyImage].

<br><br>

# Tab 2: Background

## Select Strip

If there is more than one strip on the analysed image, one first needs to select, which strip shall be analysed.

## Optional: in case of color images

If the image is a color image, it will be transformed to grayscale. By default, we apply the luminance approach, which converts color to grayscale preserving luminance; that is, the grayscale values are obtained by

$$  0.2126 * R + 0.7152 * G + 0.0722 * B$$ where R, G, B stands for the red, green and blue channel of the color image. By selecting mode "gray", the arithmetic mean of the RGB values is used. Furthermore, the selection of mode "red", "green" or "blue" allows a color channel wise analysis of color images.

## Select threshold method and apply

There are four threshold methods, where the default is Otsu's method [@Otsu.1979]. Otsu's method returns a single intensity threshold that separates pixels into foreground and background. It is equivalent to k-means clustering of the intensity histogram [@Liu.2009]. Otsu and Li [@Li.1993, @Li.1998] are non-parametric, fully automatic algorithms that find the optimal threshold for the image. Additionally, two semi-automatic algorithms, namely Quantile and Triangle [@Zack.1977] were included, to cover different images and cases where automatic threshold results are not ideal. The quantile method is a simple method that computes the specified quantile of all pixel intensities of the segments between the lines (per strip). In most of the images we have analysed so far, Otsu's method performed very well and better than our quantile method. However, in cases where the lines are unclear and very blurred our quantile method may outperform Otsu's method. In case of the triangle method an additional offset (default = 0.2) is added to the computed threshold.

By clicking "APPLY THRESHOLD" the selected threshold method is applied to the segmented images of the selected strip. The upper plots show the pixels that have intensities above the background. The lower plots show the images after background subtraction; see the screenshot below.

<p>![](BackgroundCorrection.png){width="100%"}</p>

The thresholds (top of the page) as well as calculated mean and median intensities of the lines (bottom of the page) in order from top of the strip to the bottom of the strip are shown as well.

## Add to intensity data

Clicking "ADD TO INTENSITY DATA" adds the mean and median of the background subtracted intensities of the pixels with intensities above the threshold to the data. Now, one can proceed with the second strip of the image, use a different color conversion mode or go back to Tab 1 and load the next image. When all strips and images are processed one can proceed with "SWITCH TO INTENSITY DATA".

## Switch To Intensity Data

Clicking "SWITCH TO INTENSITY DATA" changes the Tab to Tab 3.

<br><br>

# Tab 3: Intensity Data

Here, the latest version of the generated intensity data is shown in the Tab.

## Download data

By clicking "DOWNLOAD DATA" the data can be saved as a standard csv file.

## Restart with new data

We also provide a "DELETE DATA" button for restarting with new images or uploading an already existing dataset.

## Upload existing data

Instead of generating new data, one can also upload already existing intensity data, that can also be generated with a different software. This step can also be seen as a second entry point of the app. The screenshot below shows an example of data that was generated with the app saved and then loaded again.

<p>![](IntensityData.png){width="100%"}</p>

## Switch To Experiment Info

Clicking "SWITCH TO EXPERIMENT INFO" changes the Tab to Tab 4, where information about the experiment can be loaded.

<br><br>

# Tab 4: Experiment Info

## Upload Experiment Info or upload existing merged data and go to calibration

One first can either upload information about the experiment in form of a .csv file or upload already merged data (intensity data merged with experiment information). An example of a table with information about the experiment is shown in the screenshot below.

<p>![](ExperimentData.png){width="100%"}</p>

This is a third optional starting point of the app. Here, one can also directly start with already merged data and proceed with the calibration analysis.

## Select ID columns and merge datasets

For merging the intensity data with the information about the experiment, one has to specify the names of the columns on which the merge should take place. The default is "File", as a "File" column is generated when the intensity data is computed with the app. By clicking "MERGE WITH INTENSITY DATA" the two datasets will be merged. An example of a table with merged data is shown in the screenshot below.

<p>![](ExperimentMergedData.png){width="100%"}</p>

## Download data

The merged data can be downloaded as a .csv file by clicking "DOWNLOAD DATA".

## Restart with new data

We again provide a "DELETE DATA" button for restarting with new images, uploading new intensity data on Tab 3 or uploading an already existing merged dataset.

## Prepare Calibration

Clicking "PREPARE CALIBRATION" changes the Tab to Tab 5, where the data can be further preprocessed and the calibration analysis can be started.

<br><br>

# Tab 5: Calibration

## Specify a folder for the analysis results

In the first step, a folder for the analysis results should be specified. If no folder is provided, a new folder "LFApp" will be generated in the home directory of the user.

## Upload existing preprocessed data

Instead of generating the preprocessed data with our app, one can also upload preprocessed data and perform the calibration analysis on them. An example of uploaded preprocessed data that were generated with our app and saved as a .csv file is shown in the following screenshot.

<p>![](Opt1Calibration.png){width="100%"}</p>

## Optional: average technical replicates

If the merged data includes technical replicates, these replicates can be averaged either by the arithmetic mean or the median. For this purpose the name of the column, which enables the identification of the samples is required. By default we assume that this column is called "Sample". In case there is more than one analyte/color per line/band, one should set "Number of analytes/colors per line" to the respective number. If this number is larger than 1, an additional field will be shown, where one should enter the name of the column including the analyte/color information.

## Optional: reshape data from long to wide

If multiple images were taken for a single line or band using different color channels, it can be useful to combine this information into a single calibration model. In such cases, the data should be reshaped from a long format (where color-specific measurement are stored in rows) to wide format (where they are stored in columns). For this transformation, one has to specify a column that identifies each sample and another column that defines the attribute used for reshaping. By default, the sample identifier column is assumed to be named "Sample", and the reshaping attribute column is assumed to be "Color".

After reshaping, measurement columns such as `Mean1` and `Median1` are expanded to include the reshaping attribute information, resulting in columns like `Mean1.green`, `Median1.green`, `Mean1.red` and `Median1.red`.

<p>![](Opt2Calibration.png){width="100%"}</p>

## Download data

By clicking "DOWNLOAD DATA", the data used for the calibration can be downloaded as .csv file.

## Calibration by linear, local polynomial or generalized additive model

In the final step of this Tab, one has to specify the calibration model. The model can the chosen via a checkbox (linear (lm), local polynomial (loess), generalized additive model (GAM)). A column for concentration values has to be selected. The concentration values can be logarithmized. In addition, one has to specify a response variable.

<p>![](Calibration.png){width="100%"}</p>

If only a subset of the data shall be used for the calibration analysis, an logical R expression can be specified in the field below "Optional: specify subset (logical R expression)". This expression will be applied to select the respective subset.

## Run calibration analysis

By pressing "RUN CALIBRATION ANALYSIS" the RData-file "CalibrationData.RData" will be generated and saved in the specified analysis folder. Next, the R markdown file "CalibrationAnalysis(model).Rmd", where model = lm, loess, gam, will be copied to the same folder and will be rendered into file "CalibrationAnalysis(model).html" by using R package "rmarkdown" [@rmarkdown]. The results of the calibration analysis will be saved in file "CalibrationResults.RData" and will be used to generated a brief overview of the results that will be shown on Tab 6. All files will remain in the folder selected for the analysis results. That is, one may use and modify these files for an extended analysis. In addition, the fitted values for the concentrations are added to data used for the calibration. That is, one can also run several different calibration models, download the calibration data and use the data to compare the results of different calibration models.

<br><br>

# Tab 6: Results

This Tab shows a brief overview of the analysis results as shown in the following screenshot.

<p>![](Results.png){width="100%"}</p>

## Open analysis report

By clicking "Open" the full report of the calibration analysis will be opened in the standard browser. The first part of the report is visible in the following screenshot.

<p>![](AnalysisReport.png){width="100%"}</p>

<br><br>

# Tab 7: Quantification

The final tab of our application enables the quantification of intensity values with the help of a calibration model. You can upload an existing model from a previous calibration analysis and use it to predict the concentrations for the intensity values acquired beforehand by clicking "PREDICT". Via the "DOWNLOAD DATA" button the data table can be saved as a .csv file containing the intensity values as well as the predicted concentrations.

<p>![](Quantification.png){width="100%"}</p>

<br><br>

# Required R packages

Our app requires the following R packages to work properly:

- stats: The R Stats Package [@R2020]
- mgcv: Mixed GAM Computation Vehicle with Automatic Smoothness Estimation [@mgcv]
- shiny: Web Application Framework for R [@Shiny2020]
- shinyjs: Easily Improve the User Experience of Your Shiny Apps in Seconds [@shinyjs.2020]
- shinythemes: Themes for Shiny [@shinythemes.2018]
- fs: Cross-Platform File System Operations Based on 'libuv' [@fs.2020]
- shinyFiles: A Server-Side File System Viewer for Shiny [@shinyfiles.2020]
- Bioconductor package EBImage: R package for image processing with applications to cellular phenotypes [@Pau.2010]
- DT: A Wrapper of the JavaScript Library 'DataTables' [@DT.2020]
- rmarkdown: Dynamic Documents for R [@rmarkdown]
- ggplot2: Elegant Graphics for Data Analysis [@Wickham.2016]

<br><br>

# References

\name{run_functions}
\alias{run_analysis}
\alias{run_cal}
\alias{run_core}
\alias{run_quan}
\alias{run_mobile_analysis}
\alias{run_mobile_cal}
\alias{run_mobile_core}
\alias{run_mobile_quan}
\title{Run Analysis Shiny Apps}
\description{
  Function start the Analysis Shiny App.
}
\usage{
run_analysis()
run_cal()
run_core()
run_quan()
run_mobile_analysis()
run_mobile_cal()
run_mobile_core()
run_mobile_quan()
}
\details{
  The functions start the various shiny apps included in the package.
}
\value{
  An object that represents the app. Printing the object will run the app.
}
\author{Filip Paskali \email{F.Paskali@hs-furtwangen.de}, 
        Weronika Schary \email{W.Schary@hs-furtwangen.de}, 
        Matthias Kohl \email{Matthias.Kohl@stamats.de}}
\examples{
if(interactive()){
  ## start full analysis app
  run_analysis()
  ## start mobile version of full analysis app
  run_mobile_analysis()
}
}
\keyword{dynamic}

\name{triangle}
\alias{triangle}
\title{Triangle Thresholding Algorithm}
\description{
  The function computes a background threshold of an image using the triangle 
  algorithm.
}
\usage{
triangle(image, offset = 0.2, breaks = 256)
}
\arguments{
  \item{image}{ object of class \code{Image} from package EBImage. }
  \item{offset}{ numeric, additional offset added to the computed threshold. }
  \item{breaks}{ integer, number of breaks used in the histogram. }
}
\details{
  The Triangle method is based on the histogram of the intensities. Based on 
  the range of intensities and the maximum peak a threshold is determined. 
  The method was proposed in Zack et al. (1977).
}
\value{
  \code{numeric} vector with the computed threshold.
}
\references{
  G.W. Zack, W. E. Rogers, and S. A. Latt (1977). Automatic measurement of sister 
  chromatid exchange frequency. \emph{The journal of histochemistry and cytochemistry: 
  official journal of the Histochemistry Society} \bold{25} (7): 741-53. 
  https://doi.org/10.1177/25.7.70454.
}
\author{Matthias Kohl \email{Matthias.Kohl@stamats.de}}
\examples{
library(EBImage)
x <- readImage(system.file("images", "sample.TIF", package="LFApp"))
triangle(x)
}
\keyword{univar}

\name{LFApp-package}
\alias{LFApp-package}
\alias{LFApp}
\docType{package}
\title{
Shiny Apps for Lateral Flow Assays.
}
\description{
Shiny apps for the quantitative analysis of images from lateral flow assays (LFAs). 
The images are segmented and background corrected and color intensities are extracted. 
The apps can be used to import and export intensity data and to calibrate LFAs by 
means of linear, loess, or gam models. The calibration models can further be saved 
and applied to intensity data from new images for determining concentrations.
}
\details{
library(LFApp)
}
\author{ Filip Paskali, Weronika Schary, Matthias Kohl

Maintainer: Filip Paskali  \email{Filip.Paskali@gmail.com}}
%\references{
%~~ Literature or other references for background information ~~
%}
%~~ Optionally other keywords from doc/KEYWORDS, one per line
%\seealso{
%~~ Optional links to other man pages, e.g. ~~
%~~ \code{\link[<pkg>:<pkg>-package]{<pkg>}} ~~
%}
%\examples{
%~~ simple examples of the most important functions ~~
%}
\keyword{package}

\name{threshold_li}
\alias{threshold_li}
\title{Li Thresholding Algorithm}
\description{
  The function computes a background threshold of an image by using Li's 
  iterative minimum cross entropy method.
}
\usage{
threshold_li(image, tolerance = NULL, initial_guess = NULL, iter_callback = NULL)
}
\arguments{
  \item{image}{ object of class \code{Image} from package EBImage. }
  \item{tolerance}{ optional tolerance threshold.}
  \item{initial_guess}{ optional initial value for the minimization. }
  \item{iter_callback}{ optional function applied to the minimization criterion. }
}
\details{
  For more details about the method see Li and Lee (1993) as well as 
  Li and Tam (1998).
}
\value{
  \code{numeric} vector with the computed threshold.
}
\references{
  C.H. Li and C.K. Lee (1993). Minimum cross entropy thresholding. 
  \emph{Pattern Recognition} \bold{26} (4): 617-25. 
  https://doi.org/10.1016/0031-3203(93)90115-D.

  C.H. Li and P.K.S. Tam (1998). An iterative algorithm for minimum cross entropy 
  thresholding. \emph{Pattern Recognition Letters} \bold{19} (8): 771-76. 
  https://doi.org/10.1016/S0167-8655(98)00057-9.

}
\author{Filip Paskali \email{Filip.Paskali@gmail.de}}
\examples{
library(EBImage)
x <- readImage(system.file("images", "sample.TIF", package="LFApp"))
threshold_li(x)
}
\keyword{univar}
